@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: daemon.c 10594 2010-05-01 13:45:03Z charles $
+ * $Id: daemon.c 10659 2010-05-18 22:25:14Z Longinus00 $
  */
 
 #include <errno.h>
@@ -43,6 +43,7 @@
 static tr_bool paused = FALSE;
 static tr_bool closing = FALSE;
 static tr_session * mySession = NULL;
+static const char * pid_filename = NULL;
 
 /***
 ****  Config File
@@ -85,6 +86,8 @@ static const struct tr_option options[] =
     { 800, "paused", "Pause all torrents on startup", NULL, 0, NULL },
     { 'o', "dht", "Enable distributed hash tables (DHT)", "o", 0, NULL },
     { 'O', "no-dht", "Disable distributed hash tables (DHT)", "O", 0, NULL },
+    { 'y', "lpd", "Enable local peer discovery (LPD)", "y", 0, NULL },
+    { 'Y', "no-lpd", "Disable local peer discovery (LPD)", "Y", 0, NULL },
     { 'P', "peerport", "Port for incoming peers (Default: " TR_DEFAULT_PEER_PORT_STR ")", "P", 1, "<port>" },
     { 'm', "portmap", "Enable portmapping via NAT-PMP or UPnP", "m", 0, NULL },
     { 'M', "no-portmap", "Disable portmapping", "M", 0, NULL },
@@ -98,6 +101,7 @@ static const struct tr_option options[] =
     { 'r', "rpc-bind-address", "Where to listen for RPC connections", "r", 1, "<ipv4 address>" },
     { 953, "global-seedratio", "All torrents, unless overridden by a per-torrent setting, should seed until a specific ratio", "gsr", 1, "ratio" },
     { 954, "no-global-seedratio", "All torrents, unless overridden by a per-torrent setting, should seed regardless of ratio", "GSR", 0, NULL },
+    { 'x', "pid-file", "Enable PID file", "x", 1, "<pid-file>" },
     { 0, NULL, NULL, NULL, 0, NULL }
 };
 
@@ -218,18 +222,34 @@ getConfigDir( int argc, const char ** argv )
 static void
 onFileAdded( tr_session * session, const char * dir, const char * file )
 {
-    if( strstr( file, ".torrent" ) != NULL )
+    char * filename = tr_buildPath( dir, file, NULL );
+    tr_ctor * ctor = tr_ctorNew( session );
+    int err = tr_ctorSetMetainfoFromFile( ctor, filename );
+
+    if( !err )
     {
-        char * filename = tr_buildPath( dir, file, NULL );
-        tr_ctor * ctor = tr_ctorNew( session );
+        tr_torrentNew( ctor, &err );
 
-        int err = tr_ctorSetMetainfoFromFile( ctor, filename );
-        if( !err )
-            tr_torrentNew( ctor, &err );
+        if( err == TR_PARSE_ERR )
+            tr_err( "Error parsing .torrent file \"%s\"", file );
+        else
+        {
+            tr_bool trash = FALSE;
+            int test = tr_ctorGetDeleteSource( ctor, &trash );
 
-        tr_ctorFree( ctor );
-        tr_free( filename );
+            tr_inf( "Parsing .torrent file successful \"%s\"", file );
+
+            if( !test && trash )
+            {
+                tr_inf( "Deleting input .torrent file \"%s\"", file );
+                if( remove( filename ) )
+                    tr_err( "Error deleting .torrent file: %s", tr_strerror( errno ) );
+            }
+        }
     }
+
+    tr_ctorFree( ctor );
+    tr_free( filename );
 }
 
 static void
@@ -289,6 +309,7 @@ main( int argc, char ** argv )
     const char * configDir = NULL;
     dtr_watchdir * watchdir = NULL;
     FILE * logfile = NULL;
+    tr_bool pidfile_created = FALSE;
 
     signal( SIGINT, gotsig );
     signal( SIGTERM, gotsig );
@@ -318,13 +339,11 @@ main( int argc, char ** argv )
                       break;
             case 'C': tr_bencDictAddBool( &settings, PREF_KEY_DIR_WATCH_ENABLED, FALSE );
                       break;
-	    case 941:
-        	      tr_bencDictAddStr( &settings, TR_PREFS_KEY_INCOMPLETE_DIR, optarg );
-		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, TRUE );
-		      break;
-	    case 942:
-		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, FALSE );
-		      break;
+            case 941: tr_bencDictAddStr( &settings, TR_PREFS_KEY_INCOMPLETE_DIR, optarg );
+                      tr_bencDictAddBool( &settings, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, TRUE );
+                      break;
+            case 942: tr_bencDictAddBool( &settings, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, FALSE );
+                      break;
             case 'd': dumpSettings = TRUE;
                       break;
             case 'e': logfile = fopen( optarg, "a+" );
@@ -335,15 +354,13 @@ main( int argc, char ** argv )
                       break;
             case 'g': /* handled above */
                       break;
-	    case 'V': /* version */
-		      fprintf(stderr, "Transmission %s\n", LONG_VERSION_STRING);
-		      exit( 0 );
-	    case 'o':
-		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_DHT_ENABLED, TRUE );
-		      break;
-	    case 'O':
-		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_DHT_ENABLED, FALSE );
-		      break;
+            case 'V': /* version */
+                      fprintf(stderr, "Transmission %s\n", LONG_VERSION_STRING);
+                      exit( 0 );
+            case 'o': tr_bencDictAddBool( &settings, TR_PREFS_KEY_DHT_ENABLED, TRUE );
+                      break;
+            case 'O': tr_bencDictAddBool( &settings, TR_PREFS_KEY_DHT_ENABLED, FALSE );
+                      break;
             case 'p': tr_bencDictAddInt( &settings, TR_PREFS_KEY_RPC_PORT, atoi( optarg ) );
                       break;
             case 't': tr_bencDictAddBool( &settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, TRUE );
@@ -374,22 +391,23 @@ main( int argc, char ** argv )
                       break;
             case 912: tr_bencDictAddInt( &settings, TR_PREFS_KEY_ENCRYPTION, TR_CLEAR_PREFERRED );
                       break;
-            case 'i':
-                      tr_bencDictAddStr( &settings, TR_PREFS_KEY_BIND_ADDRESS_IPV4, optarg );
+            case 'i': tr_bencDictAddStr( &settings, TR_PREFS_KEY_BIND_ADDRESS_IPV4, optarg );
                       break;
-            case 'I':
-                      tr_bencDictAddStr( &settings, TR_PREFS_KEY_BIND_ADDRESS_IPV6, optarg );
+            case 'I': tr_bencDictAddStr( &settings, TR_PREFS_KEY_BIND_ADDRESS_IPV6, optarg );
                       break;
-            case 'r':
-                      tr_bencDictAddStr( &settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, optarg );
+            case 'r': tr_bencDictAddStr( &settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, optarg );
+                      break;
+            case 953: tr_bencDictAddReal( &settings, TR_PREFS_KEY_RATIO, atof(optarg) );
+                      tr_bencDictAddBool( &settings, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
+                      break;
+            case 954: tr_bencDictAddBool( &settings, TR_PREFS_KEY_RATIO_ENABLED, FALSE );
+                      break;
+            case 'x': pid_filename = optarg;
+                      break;
+            case 'y': tr_bencDictAddBool( &settings, TR_PREFS_KEY_LPD_ENABLED, TRUE );
+                      break;
+            case 'Y': tr_bencDictAddBool( &settings, TR_PREFS_KEY_LPD_ENABLED, FALSE );
                       break;
-	    case 953:
-		      tr_bencDictAddReal( &settings, TR_PREFS_KEY_RATIO, atof(optarg) );
-		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
-		      break;
-	    case 954:
-		      tr_bencDictAddBool( &settings, TR_PREFS_KEY_RATIO_ENABLED, FALSE );
-		      break;
             default:  showUsage( );
                       break;
         }
@@ -425,6 +443,20 @@ main( int argc, char ** argv )
     tr_ninf( NULL, "Using settings from \"%s\"", configDir );
     tr_sessionSaveSettings( mySession, configDir, &settings );
 
+    if( pid_filename != NULL )
+    {
+        FILE * fp = fopen( pid_filename, "w+" );
+        if( fp != NULL )
+        {
+            fprintf( fp, "%d", (int)getpid() );
+            fclose( fp );
+            tr_inf( "Saved pidfile \"%s\"", pid_filename );
+            pidfile_created = TRUE;
+        }
+        else
+            tr_err( "Unable to save pidfile \"%s\": %s", pid_filename, strerror( errno ) );
+    }
+
     if( tr_bencDictFindBool( &settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, &boolVal ) && boolVal )
         tr_ninf( MY_NAME, "requiring authentication" );
 
@@ -456,7 +488,7 @@ main( int argc, char ** argv )
 
 #ifdef HAVE_SYSLOG
     if( !foreground )
-        openlog( MY_NAME, LOG_CONS, LOG_DAEMON );
+        openlog( MY_NAME, LOG_CONS|LOG_PID, LOG_DAEMON );
 #endif
 
     while( !closing ) {
@@ -481,6 +513,8 @@ main( int argc, char ** argv )
     printf( " done.\n" );
 
     /* cleanup */
+    if( pidfile_created )
+        remove( pid_filename );
     tr_bencFree( &settings );
     return 0;
 }
