@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: add-dialog.h 7658 2009-01-10 23:09:07Z charles $
+ * $Id: add-dialog.h 9868 2010-01-04 21:00:47Z charles $
  */
 
 #ifndef TR_GTK_ADD_DIALOG_H
@@ -24,4 +24,7 @@ GtkWidget* addSingleTorrentDialog( GtkWindow * parent,
 GtkWidget* addDialog( GtkWindow * parent,
                       TrCore *    core );
 
+GtkWidget* addURLDialog( GtkWindow * parent,
+                         TrCore *    core );
+
 #endif /* TR_GTK_ADD_DIALOG */
