@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 10598 2010-05-01 14:06:53Z charles $
+ * $Id: main.c 11007 2010-07-13 14:09:24Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -84,6 +84,7 @@ struct cbdata
     GSList            * dupqueue;
     GSList            * details;
     GtkTreeSelection  * sel;
+    GtkWidget         * quit_dialog;
 };
 
 /**
@@ -281,7 +282,7 @@ accumulateStatusForeach( GtkTreeModel *      model,
                          GtkTreeIter *       iter,
                          gpointer            user_data )
 {
-    int                  activity = 0;
+    int activity = 0;
     struct counts_data * counts = user_data;
 
     ++counts->totalCount;
@@ -294,6 +295,16 @@ accumulateStatusForeach( GtkTreeModel *      model,
         ++counts->activeCount;
 }
 
+static void
+getTorrentCounts( struct cbdata * data, struct counts_data * counts )
+{
+    counts->activeCount = 0;
+    counts->inactiveCount = 0;
+    counts->totalCount = 0;
+
+    gtk_tree_selection_selected_foreach( data->sel, accumulateStatusForeach, counts );
+}
+
 static void
 accumulateCanUpdateForeach( GtkTreeModel *      model,
                             GtkTreePath  * path UNUSED,
@@ -305,33 +316,30 @@ accumulateCanUpdateForeach( GtkTreeModel *      model,
     *(int*)accumulated_status |= tr_torrentCanManualUpdate( tor );
 }
 
-static void
-refreshActions( struct cbdata * data )
+static gboolean
+refreshActions( gpointer gdata )
 {
     int canUpdate;
     struct counts_data counts;
-    GtkTreeSelection * s = data->sel;
+    struct cbdata * data = gdata;
 
-    counts.activeCount = 0;
-    counts.inactiveCount = 0;
-    counts.totalCount = 0;
-    gtk_tree_selection_selected_foreach( s, accumulateStatusForeach, &counts );
+    getTorrentCounts( data, &counts );
     action_sensitize( "pause-torrent", counts.activeCount != 0 );
     action_sensitize( "start-torrent", counts.inactiveCount != 0 );
     action_sensitize( "remove-torrent", counts.totalCount != 0 );
     action_sensitize( "delete-torrent", counts.totalCount != 0 );
     action_sensitize( "verify-torrent", counts.totalCount != 0 );
+    action_sensitize( "relocate-torrent", counts.totalCount != 0 );
     action_sensitize( "show-torrent-properties", counts.totalCount != 0 );
     action_sensitize( "open-torrent-folder", counts.totalCount == 1 );
-    action_sensitize( "relocate-torrent", counts.totalCount == 1 );
     action_sensitize( "copy-magnet-link-to-clipboard", counts.totalCount == 1 );
 
     canUpdate = 0;
-    gtk_tree_selection_selected_foreach( s, accumulateCanUpdateForeach, &canUpdate );
+    gtk_tree_selection_selected_foreach( data->sel, accumulateCanUpdateForeach, &canUpdate );
     action_sensitize( "update-tracker", canUpdate != 0 );
 
     {
-        GtkTreeView *  view = gtk_tree_selection_get_tree_view( s );
+        GtkTreeView * view = gtk_tree_selection_get_tree_view( data->sel );
         GtkTreeModel * model = gtk_tree_view_get_model( view );
         const int torrentCount = gtk_tree_model_iter_n_children( model, NULL ) != 0;
         action_sensitize( "select-all", torrentCount != 0 );
@@ -339,18 +347,19 @@ refreshActions( struct cbdata * data )
     }
 
     {
-        tr_session * session = tr_core_session( data->core );
-        const int active = tr_sessionGetActiveTorrentCount( session );
-        const int total = tr_sessionCountTorrents( session );
+        const int total = tr_core_get_torrent_count( data->core );
+        const int active = tr_core_get_active_torrent_count( data->core );
         action_sensitize( "pause-all-torrents", active != 0 );
         action_sensitize( "start-all-torrents", active != total );
     }
+
+    return FALSE;
 }
 
 static void
 selectionChangedCB( GtkTreeSelection * s UNUSED, gpointer data )
 {
-    refreshActions( data );
+    gtr_idle_add( refreshActions, data );
 }
 
 static void
@@ -401,6 +410,21 @@ signal_handler( int sig )
     }
 }
 
+struct remove_torrent_idle_data
+{
+    TrCore * core;
+    int id;
+};
+
+static gboolean
+remove_torrent_idle( gpointer gdata )
+{
+    struct remove_torrent_idle_data * data = gdata;
+    tr_core_remove_torrent_from_id( data->core, data->id, FALSE );
+    g_free( data );
+    return FALSE; /* tell g_idle not to call this func twice */
+}
+
 static void
 setupsighandlers( void )
 {
@@ -414,6 +438,7 @@ onRPCChanged( tr_session            * session,
               struct tr_torrent     * tor,
               void                  * gdata )
 {
+    tr_rpc_callback_status status = TR_RPC_OK;
     struct cbdata * cbdata = gdata;
     gdk_threads_enter( );
 
@@ -423,9 +448,14 @@ onRPCChanged( tr_session            * session,
             tr_core_add_torrent( cbdata->core, tr_torrent_new_preexisting( tor ), TRUE );
             break;
 
-        case TR_RPC_TORRENT_REMOVING:
-            tr_core_torrent_destroyed( cbdata->core, tr_torrentId( tor ) );
+        case TR_RPC_TORRENT_REMOVING: {
+            struct remove_torrent_idle_data * data = g_new0( struct remove_torrent_idle_data, 1 );
+            data->id = tr_torrentId( tor );
+            data->core = cbdata->core;
+            gtr_idle_add( remove_torrent_idle, data );
+            status = TR_RPC_NOREMOVE;
             break;
+        }
 
         case TR_RPC_SESSION_CHANGED:
             tr_sessionGetSettings( session, pref_get_all( ) );
@@ -440,7 +470,7 @@ onRPCChanged( tr_session            * session,
     }
 
     gdk_threads_leave( );
-    return TR_RPC_OK;
+    return status;
 }
 
 static GSList *
@@ -551,9 +581,18 @@ main( int argc, char ** argv )
          * if that goes well, then our work is done. */
         GSList * l;
         gboolean delegated = FALSE;
+        const gboolean trash_originals = pref_flag_get( TR_PREFS_KEY_TRASH_ORIGINAL );
 
-        for( l = argfiles; l; l = l->next )
-            delegated |= gtr_dbus_add_torrent( l->data );
+        for( l=argfiles; l!=NULL; l=l->next )
+        {
+            const char * filename = l->data;
+            const gboolean added = gtr_dbus_add_torrent( filename );
+
+            if( added && trash_originals )
+                gtr_file_trash_or_remove( filename );
+
+            delegated |= added;
+        }
 
         if( delegated ) {
             g_slist_foreach( argfiles, (GFunc)g_free, NULL );
@@ -615,9 +654,15 @@ main( int argc, char ** argv )
         tr_sessionSetRPCCallback( session, onRPCChanged, cbdata );
 
         /* on startup, check & see if it's time to update the blocklist */
-        if( pref_flag_get( PREF_KEY_BLOCKLIST_UPDATES_ENABLED )
-            && ( time( NULL ) - pref_int_get( "blocklist-date" ) > ( 60 * 60 * 24 * 7 ) ) )
-                tr_core_blocklist_update( cbdata->core );
+        if( pref_flag_get( TR_PREFS_KEY_BLOCKLIST_ENABLED ) ) {
+            if( pref_flag_get( PREF_KEY_BLOCKLIST_UPDATES_ENABLED ) ) {
+                const int64_t last_time = pref_int_get( "blocklist-date" );
+                const int SECONDS_IN_A_WEEK = 7 * 24 * 60 * 60;
+                const time_t now = time( NULL );
+                if( last_time + SECONDS_IN_A_WEEK < now )
+                    tr_core_blocklist_update( cbdata->core );
+            }
+        }
 
         /* if there's no magnet link handler registered, register us */
         registerMagnetLinkHandler( );
@@ -708,7 +753,7 @@ appsetup( TrWindow *      wind,
                                                 GTK_MESSAGE_INFO,
                                                 GTK_BUTTONS_NONE,
                                                 "%s",
-            _( "Transmission is a file sharing program.  When you run a torrent, its data will be made available to others by means of upload.  And of course, any content you share is your sole responsibility.\n\nYou probably knew this, so we won't tell you again." ) );
+             _( "Transmission is a file-sharing program.  When you run a torrent, its data will be made available to others by means of upload.  You and you alone are fully responsible for exercising proper judgement and abiding by your local laws." ) );
         gtk_dialog_add_button( GTK_DIALOG( w ), GTK_STOCK_QUIT, GTK_RESPONSE_REJECT );
         gtk_dialog_add_button( GTK_DIALOG( w ), _( "I _Accept" ), GTK_RESPONSE_ACCEPT );
         gtk_dialog_set_default_response( GTK_DIALOG( w ), GTK_RESPONSE_ACCEPT );
@@ -735,15 +780,14 @@ tr_window_present( GtkWindow * window )
 }
 
 static void
-toggleMainWindow( struct cbdata * cbdata,
-                  gboolean        doPresent )
+toggleMainWindow( struct cbdata * cbdata )
 {
     GtkWindow * window = GTK_WINDOW( cbdata->wind );
     const int   doShow = cbdata->isIconified;
     static int  x = 0;
     static int  y = 0;
 
-    if( doShow || doPresent )
+    if( doShow )
     {
         cbdata->isIconified = 0;
         gtk_window_set_skip_taskbar_hint( window, FALSE );
@@ -760,6 +804,30 @@ toggleMainWindow( struct cbdata * cbdata,
     }
 }
 
+static gboolean
+shouldConfirmBeforeExiting( struct cbdata * data )
+{
+    if( !pref_flag_get( PREF_KEY_ASKQUIT ) )
+        return FALSE;
+    else {
+        struct counts_data counts;
+        getTorrentCounts( data, &counts );
+        return counts.activeCount > 0;
+    }
+}
+
+static void
+maybeaskquit( struct cbdata * cbdata )
+{
+    if( !shouldConfirmBeforeExiting( cbdata ) )
+        wannaquit( cbdata );
+    else {
+        if( cbdata->quit_dialog == NULL )
+            cbdata->quit_dialog = askquit( cbdata->core, cbdata->wind, wannaquit, cbdata );
+        gtk_window_present( GTK_WINDOW( cbdata->quit_dialog ) );
+    }
+}
+
 static gboolean
 winclose( GtkWidget * w    UNUSED,
           GdkEvent * event UNUSED,
@@ -770,7 +838,7 @@ winclose( GtkWidget * w    UNUSED,
     if( cbdata->icon != NULL )
         action_activate ( "toggle-main-window" );
     else
-        askquit( cbdata->core, cbdata->wind, wannaquit, cbdata );
+        maybeaskquit( cbdata );
 
     return TRUE; /* don't propagate event further */
 }
@@ -1125,12 +1193,10 @@ onAddTorrent( TrCore *  core,
 }
 
 static void
-prefschanged( TrCore * core UNUSED,
-              const char *  key,
-              gpointer      data )
+prefschanged( TrCore * core UNUSED, const char * key, gpointer data )
 {
-    struct cbdata  * cbdata = data;
-    tr_session     * tr     = tr_core_session( cbdata->core );
+    struct cbdata * cbdata = data;
+    tr_session * tr = tr_core_session( cbdata->core );
 
     if( !strcmp( key, TR_PREFS_KEY_ENCRYPTION ) )
     {
@@ -1202,6 +1268,10 @@ prefschanged( TrCore * core UNUSED,
     {
         tr_sessionSetDHTEnabled( tr, pref_flag_get( key ) );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_LPD_ENABLED ) )
+    {
+        tr_sessionSetLPDEnabled( tr, pref_flag_get( key ) );
+    }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_PORT ) )
     {
         tr_sessionSetRPCPort( tr, pref_int_get( key ) );
@@ -1300,6 +1370,22 @@ prefschanged( TrCore * core UNUSED,
     {
         tr_sessionSetIncompleteDirEnabled( tr, pref_flag_get( key ) );
     }
+    else if( !strcmp( key, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED ) )
+    {
+        tr_sessionSetTorrentDoneScriptEnabled( tr, pref_flag_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME ) )
+    {
+        tr_sessionSetTorrentDoneScript( tr, pref_string_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_START) )
+    {
+        tr_sessionSetPaused( tr, !pref_flag_get( key ) );
+    }
+    else if( !strcmp( key, TR_PREFS_KEY_TRASH_ORIGINAL ) )
+    {
+        tr_sessionSetDeleteSource( tr, pref_flag_get( key ) );
+    }
 }
 
 static gboolean
@@ -1431,28 +1517,28 @@ msgwinclosed( void )
 }
 
 static void
-accumulateSelectedTorrents( GtkTreeModel *      model,
+accumulateSelectedTorrents( GtkTreeModel * model,
                             GtkTreePath  * path UNUSED,
-                            GtkTreeIter *       iter,
-                            gpointer            gdata )
+                            GtkTreeIter  * iter,
+                            gpointer       gdata )
 {
-    GSList **   data = ( GSList** ) gdata;
-    TrTorrent * tor = NULL;
+    GSList ** data = ( GSList** ) gdata;
+    TrTorrent * gtor = NULL;
 
-    gtk_tree_model_get( model, iter, MC_TORRENT, &tor, -1 );
-    *data = g_slist_prepend( *data, tor );
+    gtk_tree_model_get( model, iter, MC_TORRENT, &gtor, -1 );
+    *data = g_slist_prepend( *data, gtor );
+    g_object_unref( G_OBJECT( gtor ) );
 }
 
 static void
-removeSelected( struct cbdata * data,
-                gboolean        delete_files )
+removeSelected( struct cbdata * data, gboolean delete_files )
 {
-    GSList *           l = NULL;
+    GSList * l = NULL;
     GtkTreeSelection * s = tr_window_get_selection( data->wind );
 
     gtk_tree_selection_selected_foreach( s, accumulateSelectedTorrents, &l );
-    gtk_tree_selection_unselect_all( s );
-    if( l )
+
+    if( l != NULL )
     {
         l = g_slist_reverse( l );
         confirmRemove( data->wind, data->core, l, delete_files );
@@ -1571,11 +1657,11 @@ doAction( const char * action_name, gpointer user_data )
     }
     else if( !strcmp( action_name, "relocate-torrent" ) )
     {
-        tr_torrent * tor = getFirstSelectedTorrent( data );
-        if( tor != NULL )
+        GSList * ids = getSelectedTorrentIds( data );
+        if( ids != NULL )
         {
             GtkWindow * parent = GTK_WINDOW( data->wind );
-            GtkWidget * w = gtr_relocate_dialog_new( parent, tor );
+            GtkWidget * w = gtr_relocate_dialog_new( parent, data->core, ids );
             gtk_widget_show( w );
         }
     }
@@ -1633,7 +1719,7 @@ doAction( const char * action_name, gpointer user_data )
     }
     else if( !strcmp( action_name, "quit" ) )
     {
-        askquit( data->core, data->wind, wannaquit, data );
+        maybeaskquit( data );
     }
     else if( !strcmp( action_name, "select-all" ) )
     {
@@ -1684,11 +1770,7 @@ doAction( const char * action_name, gpointer user_data )
     }
     else if( !strcmp( action_name, "toggle-main-window" ) )
     {
-        toggleMainWindow( data, FALSE );
-    }
-    else if( !strcmp( action_name, "present-main-window" ) )
-    {
-        toggleMainWindow( data, TRUE );
+        toggleMainWindow( data );
     }
     else g_error ( "Unhandled action: %s", action_name );
 
