@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-core.c 10108 2010-02-06 05:21:25Z charles $
+ * $Id: tr-core.c 10853 2010-06-25 17:49:53Z charles $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -455,11 +455,11 @@ static void
 tr_core_apply_defaults( tr_ctor * ctor )
 {
     if( tr_ctorGetPaused( ctor, TR_FORCE, NULL ) )
-        tr_ctorSetPaused( ctor, TR_FORCE, !pref_flag_get( PREF_KEY_START ) );
+        tr_ctorSetPaused( ctor, TR_FORCE, !pref_flag_get( TR_PREFS_KEY_START ) );
 
     if( tr_ctorGetDeleteSource( ctor, NULL ) )
         tr_ctorSetDeleteSource( ctor,
-                               pref_flag_get( PREF_KEY_TRASH_ORIGINAL ) );
+                               pref_flag_get( TR_PREFS_KEY_TRASH_ORIGINAL ) );
 
     if( tr_ctorGetPeerLimit( ctor, TR_FORCE, NULL ) )
         tr_ctorSetPeerLimit( ctor, TR_FORCE,
@@ -948,16 +948,17 @@ add_ctor( TrCore * core, tr_ctor * ctor, gboolean doPrompt, gboolean doNotify )
 void
 tr_core_add_ctor( TrCore * core, tr_ctor * ctor )
 {
-    const gboolean doStart = pref_flag_get( PREF_KEY_START );
     const gboolean doPrompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
+    const gboolean doNotify = FALSE;
     tr_core_apply_defaults( ctor );
-    add_ctor( core, ctor, doStart, doPrompt );
+    add_ctor( core, ctor, doPrompt, doNotify );
 }
 
 /* invoked remotely via dbus. */
 gboolean
 tr_core_add_metainfo( TrCore      * core,
                       const char  * payload,
+                      const char  * filename,
                       gboolean    * setme_handled,
                       GError     ** gerr UNUSED )
 {
@@ -972,23 +973,38 @@ tr_core_add_metainfo( TrCore      * core,
         tr_core_add_from_url( core, payload );
         *setme_handled = TRUE;
     }
-    else /* base64-encoded metainfo */
+    else
     {
-        int file_length;
         tr_ctor * ctor;
-        char * file_contents;
-        gboolean do_prompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
+        gboolean has_metainfo = FALSE;
+        const gboolean do_prompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
 
+        /* create the constructor */
         ctor = tr_ctorNew( session );
         tr_core_apply_defaults( ctor );
 
-        file_contents = tr_base64_decode( payload, -1, &file_length );
-        tr_ctorSetMetainfo( ctor, (const uint8_t*)file_contents, file_length );
-        add_ctor( core, ctor, do_prompt, TRUE );
+        if( !has_metainfo && g_file_test( filename, G_FILE_TEST_IS_REGULAR ) )
+        {
+            /* set the metainfo from a local file */
+            has_metainfo = !tr_ctorSetMetainfoFromFile( ctor, filename );
+        }
 
-        tr_free( file_contents );
-        tr_core_torrents_added( core );
-        *setme_handled = TRUE;
+        if( !has_metainfo )
+        {
+            /* base64-encoded metainfo */
+            int file_length;
+            char * file_contents = tr_base64_decode( payload, -1, &file_length );
+            has_metainfo = !tr_ctorSetMetainfo( ctor, (const uint8_t*)file_contents, file_length );
+            tr_free( file_contents );
+        }
+
+        if( has_metainfo )
+        {
+            add_ctor( core, ctor, do_prompt, TRUE );
+            tr_core_torrents_added( core );
+        }
+
+        *setme_handled = has_metainfo;
     }
 
     return TRUE;
@@ -1002,40 +1018,44 @@ struct url_dialog_data
 {
     TrCore * core;
     tr_ctor * ctor;
-    GtkDialog * dialog;
+    char * url;
+    long response_code;
 };
 
 static gboolean
 onURLDoneIdle( gpointer vdata )
 {
     struct url_dialog_data * data = vdata;
-    tr_core_add_ctor( data->core, data->ctor );
+    const gboolean doPrompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT ); 
+    const gboolean doNotify = FALSE; 
+    const int err = add_ctor( data->core, data->ctor, doPrompt, doNotify ); 
+
+    if( err == TR_PARSE_ERR ) 
+        tr_core_errsig( data->core, TR_PARSE_ERR, data->url ); 
+    
+     tr_core_torrents_added( data->core ); 
+
+    /* cleanup */
+    g_free( data->url );
     g_free( data );
     return FALSE;
 }
 
 static void
 onURLDone( tr_session       * session,
-           long               response_code UNUSED,
+           long               response_code,
            const void       * response,
            size_t             response_byte_count,
            void             * vdata )
 {
     struct url_dialog_data * data = vdata;
-    tr_ctor * ctor = tr_ctorNew( session );
 
-    /* FIME: error dialog */
+    data->response_code = response_code; 
+    data->ctor = tr_ctorNew( session ); 
+    tr_core_apply_defaults( data->ctor ); 
+    tr_ctorSetMetainfo( data->ctor, response, response_byte_count ); 
 
-    if( tr_ctorSetMetainfo( ctor, response, response_byte_count ) )
-    {
-        tr_ctorFree( ctor );
-        g_free( data );
-    }
-    else /* move the work back to the gtk thread */
-    {
-        data->ctor = ctor;
-        gtr_idle_add( onURLDoneIdle, data );
-    }
+    gtr_idle_add( onURLDoneIdle, data ); 
 }
 
 void
@@ -1055,20 +1075,12 @@ tr_core_add_from_url( TrCore * core, const char * url )
 
         err = tr_ctorSetMetainfoFromMagnetLink( ctor, url );
 
-        if( err )
-        {
+        if( !err )
+            tr_core_add_ctor( core, ctor );
+        else {
             gtr_unrecognized_url_dialog( NULL, url );
             tr_ctorFree( ctor );
         }
-        else
-        {
-            tr_session * session = tr_core_session( core );
-            TrTorrent * gtor = tr_torrent_new_ctor( session, ctor, &err );
-            if( !err )
-                tr_core_add_torrent( core, gtor, FALSE );
-            else
-                g_message( "tr_torrent_new_ctor err %d", err );
-        }
 
         g_free( tmp );
     }
@@ -1076,6 +1088,7 @@ tr_core_add_from_url( TrCore * core, const char * url )
     {
         struct url_dialog_data * data = g_new( struct url_dialog_data, 1 );
         data->core = core;
+        data->url = g_strdup( url );
         tr_webRun( session, url, NULL, onURLDone, data );
     }
 }
@@ -1124,7 +1137,10 @@ tr_core_present_window( TrCore      * core UNUSED,
                         gboolean *         success,
                         GError     ** err  UNUSED )
 {
-    action_activate( "present-main-window" );
+    /* Setting the toggle-main-window GtkCheckMenuItem to
+       make sure its state is correctly set */
+    action_toggle( "toggle-main-window", TRUE);
+    
     *success = TRUE;
     return TRUE;
 }
@@ -1136,7 +1152,7 @@ tr_core_add_list( TrCore       * core,
                   pref_flag_t    prompt,
                   gboolean       doNotify )
 {
-    const gboolean doStart = pref_flag_eval( start, PREF_KEY_START );
+    const gboolean doStart = pref_flag_eval( start, TR_PREFS_KEY_START );
     const gboolean doPrompt = pref_flag_eval( prompt, PREF_KEY_OPTIONS_PROMPT );
     GSList * l;
 
@@ -1178,48 +1194,40 @@ findTorrentInModel( TrCore *      core,
 }
 
 void
-tr_core_torrent_destroyed( TrCore * core,
-                           int      id )
+tr_core_remove_torrent( TrCore * core, TrTorrent * gtor, gboolean deleteFiles )
+{
+    const tr_torrent * tor = tr_torrent_handle( gtor );
+
+    if( tor != NULL )
+        tr_core_remove_torrent_from_id( core, tr_torrentId( tor ), deleteFiles );
+}
+
+void
+tr_core_remove_torrent_from_id( TrCore * core, int id, gboolean deleteFiles )
 {
     GtkTreeIter iter;
 
     if( findTorrentInModel( core, id, &iter ) )
     {
-        TrTorrent * gtor;
+        TrTorrent * gtor = NULL;
+        tr_torrent * tor = NULL;
         GtkTreeModel * model = tr_core_model( core );
-        gtk_tree_model_get( model, &iter, MC_TORRENT, &gtor, -1 );
-        tr_torrent_clear( gtor );
+
+        gtk_tree_model_get( model, &iter, MC_TORRENT, &gtor,
+                                          MC_TORRENT_RAW, &tor,
+                                          -1 );
+
+        /* remove from the gui */
         gtk_list_store_remove( GTK_LIST_STORE( model ), &iter );
-        g_object_unref( G_OBJECT( gtor ) );
-    }
-}
 
-void
-tr_core_remove_torrent( TrCore *    core,
-                        TrTorrent * gtor,
-                        int         deleteFiles )
-{
-    const tr_torrent * tor = tr_torrent_handle( gtor );
+        /* maybe delete the downloaded files */
+        if( deleteFiles )
+            tr_torrentDeleteLocalData( tor, gtr_file_trash_or_remove );
 
-    if( tor )
-    {
-        int         id = tr_torrentId( tor );
-        GtkTreeIter iter;
-        if( findTorrentInModel( core, id, &iter ) )
-        {
-            GtkTreeModel * model = tr_core_model( core );
-
-            /* remove from the gui */
-            gtk_list_store_remove( GTK_LIST_STORE( model ), &iter );
-
-            /* maybe delete the downloaded files */
-            if( deleteFiles )
-                tr_torrent_delete_files( gtor );
-
-            /* remove the torrent */
-            tr_torrent_set_remove_flag( gtor, TRUE );
-            g_object_unref( G_OBJECT( gtor ) );
-        }
+        /* remove the torrent */
+        tr_torrent_set_remove_flag( gtor, TRUE );
+        gtr_warn_if_fail( G_OBJECT( gtor )->ref_count == 1 );
+        g_object_unref( G_OBJECT( gtor ) ); /* remove the last refcount */
     }
 }
 
@@ -1422,15 +1430,9 @@ tr_core_set_hibernation_allowed( TrCore * core,
 static void
 maybeInhibitHibernation( TrCore * core )
 {
-    gboolean inhibit = pref_flag_get( PREF_KEY_INHIBIT_HIBERNATION );
-
-    /* always allow hibernation when all the torrents are paused */
-    if( inhibit ) {
-        tr_session * session = tr_core_session( core );
-
-        if( tr_sessionGetActiveTorrentCount( session ) == 0 )
-            inhibit = FALSE;
-    }
+    /* inhibit if it's enabled *AND* all the torrents are paused */
+    const gboolean inhibit = pref_flag_get( PREF_KEY_INHIBIT_HIBERNATION )
+                          && ( tr_core_get_active_torrent_count( core ) == 0 );
 
     tr_core_set_hibernation_allowed( core, !inhibit );
 }
@@ -1666,3 +1668,55 @@ tr_core_exec( TrCore * core, const tr_benc * top )
     tr_core_exec_json( core, json );
     tr_free( json );
 }
+
+/***
+****
+***/
+
+void
+tr_core_torrent_changed( TrCore * core, int id )
+{
+    GtkTreeIter iter;
+    GtkTreeModel * model = tr_core_model( core );
+
+    if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+    {
+        tr_torrent * tor;
+        gtk_tree_model_get( model, &iter, MC_TORRENT_RAW, &tor, -1 );
+        if( tr_torrentId( tor ) == id )
+        {
+            GtkTreePath * path = gtk_tree_model_get_path( model, &iter );
+            gtk_tree_model_row_changed( model, path, &iter );
+            gtk_tree_path_free( path );
+            break;
+        }
+    }
+    while( gtk_tree_model_iter_next( model, &iter ) );
+}
+
+int
+tr_core_get_torrent_count( TrCore * core )
+{
+    return gtk_tree_model_iter_n_children( tr_core_model( core ), NULL );
+}
+
+int
+tr_core_get_active_torrent_count( TrCore * core )
+{
+    GtkTreeIter iter;
+    GtkTreeModel * model = tr_core_model( core );
+    int activeCount = 0;
+
+    if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+    {
+        int activity;
+        gtk_tree_model_get( model, &iter, MC_ACTIVITY, &activity, -1 );
+
+        if( activity != TR_STATUS_STOPPED )
+            ++activeCount;
+    }
+    while( gtk_tree_model_iter_next( model, &iter ) );
+
+    return activeCount;
+}
+
