@@ -7,21 +7,19 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: clients.c 10016 2010-01-25 05:06:54Z livings124 $
+ * $Id: clients.c 10520 2010-04-23 16:35:11Z charles $
  */
 
 /* thanks amc1! */
 
-#include <ctype.h> /* isprint */
+#include <ctype.h> /* isprint() */
 #include <stdio.h>
-#include <stdlib.h> /* strtol */
+#include <stdlib.h> /* strtol() */
 #include <string.h>
 
-#include <event.h> /* evbuffer */
-
 #include "transmission.h"
 #include "clients.h"
-#include "utils.h"
+#include "utils.h" /* tr_snprintf(), tr_strlcpy() */
 
 static int
 charint( char ch )
