@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.h 7455 2008-12-22 00:51:14Z charles $
+ * $Id: handshake.h 10500 2010-04-20 21:54:03Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -20,16 +20,24 @@
 #include "transmission.h"
 #include "net.h"
 
+/** @addtogroup peers Peers
+    @{ */
+
 struct tr_peerIo;
+
+/** @brief opaque struct holding hanshake state information.
+           freed when the handshake is completed. */
 typedef struct tr_handshake tr_handshake;
 
 /* returns true on success, false on error */
 typedef tr_bool ( *handshakeDoneCB )( struct tr_handshake * handshake,
                                       struct tr_peerIo *    io,
-                                      int                   isConnected,
+                                      tr_bool               readAnythingFromPeer,
+                                      tr_bool               isConnected,
                                       const uint8_t *       peerId,
                                       void *                userData );
 
+/** @brief instantiate a new handshake */
 tr_handshake *         tr_handshakeNew( struct tr_peerIo * io,
                                         tr_encryption_mode encryptionMode,
                                         handshakeDoneCB    doneCB,
@@ -38,8 +46,6 @@ tr_handshake *         tr_handshakeNew( struct tr_peerIo * io,
 const tr_address *     tr_handshakeGetAddr( const struct tr_handshake  * handshake,
                                             tr_port                    * port );
 
-void                   tr_handshakeFree( tr_handshake * handshake );
-
 void                   tr_handshakeAbort( tr_handshake * handshake );
 
 struct tr_peerIo*      tr_handshakeGetIO( tr_handshake * handshake );
@@ -47,4 +53,5 @@ struct tr_peerIo*      tr_handshakeGetIO( tr_handshake * handshake );
 struct tr_peerIo*      tr_handshakeStealIO( tr_handshake * handshake );
 
 
+/** @} */
 #endif
