@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: list.h 9891 2010-01-06 00:18:33Z charles $
+ * $Id: list.h 10524 2010-04-23 23:45:44Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -101,51 +101,6 @@ tr_list* tr_list_find( tr_list *         list,
                        TrListCompareFunc compare_func );
 
 
-/** @brief Double-linked list with easy memory management and fast insert/remove operations */
-struct __tr_list
-{
-    struct __tr_list * next, * prev;
-};
-
-/**
- * @brief Given a __tr_list node that's embedded in a struct, returns a pointer to the struct.
- * @param ptr     pointer to the embedded __tr_list
- * @param type    struct type that has contains the __tr_list
- * @param field   the name of the struct's _tr_list field
- */
-#define __tr_list_entry(ptr,type,field) ((type*) (((char*)ptr) - offsetof(type,field)))
-
-typedef int  ( *__tr_list_cmp_t ) ( const void * a, const void * b );
-typedef void ( *__tr_list_free_t )( void * );
-
-
-/** @brief Init @head as an empty list. */
-static inline void
-__tr_list_init( struct __tr_list * head )
-{
-    head->next = head->prev = head;
-}
-
-
-/** @brief Insert @list between @prev and @next. */
-void
-__tr_list_insert( struct __tr_list * list,
-                  struct __tr_list * prev,
-                  struct __tr_list * next);
-
-/** @brief Append @list to the end of @head. */
-static inline void
-__tr_list_append( struct __tr_list * head, struct __tr_list * list)
-{
-    __tr_list_insert( list, head->prev, head );
-}
-
-/** @brief Remove @head from the list it is in. */
-void __tr_list_remove( struct __tr_list * head );
-
-/** @brief Destroy the list and free all nodes */
-void __tr_list_destroy( struct __tr_list * head, __tr_list_free_t func );
-
 /* @} */
 #endif /* TR_LIST_H */
 
