@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,11 +7,11 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-common.h 8561 2009-05-29 19:17:12Z charles $
+ * $Id: peer-common.h 10694 2010-05-26 15:23:21Z charles $
  */
 
 #ifndef __TRANSMISSION__
-#error only libtransmission should #include this header.
+ #error only libtransmission should #include this header.
 #endif
 
 #ifndef TR_PEER_H
@@ -28,6 +28,20 @@
 
 #include "transmission.h"
 
+enum
+{
+    /** when we're making requests from another peer,
+        batch them together to send enough requests to
+        meet our bandwidth goals for the next N seconds */
+    REQUEST_BUF_SECS = 10,
+
+    /** this is the maximum size of a block request.
+        most bittorrent clients will reject requests
+        larger than this size. */
+    MAX_BLOCK_SIZE = ( 1024 * 16 )
+    
+};
+
 typedef enum
 {
     TR_ADDREQ_OK = 0,
@@ -45,15 +59,15 @@ tr_addreq_t;
 typedef enum
 {
     TR_PEER_CLIENT_GOT_BLOCK,
+    TR_PEER_CLIENT_GOT_CHOKE,
     TR_PEER_CLIENT_GOT_DATA,
     TR_PEER_CLIENT_GOT_ALLOWED_FAST,
     TR_PEER_CLIENT_GOT_SUGGEST,
+    TR_PEER_CLIENT_GOT_PORT,
+    TR_PEER_CLIENT_GOT_REJ,
     TR_PEER_PEER_GOT_DATA,
     TR_PEER_PEER_PROGRESS,
-    TR_PEER_ERROR,
-    TR_PEER_CANCEL,
-    TR_PEER_UPLOAD_ONLY,
-    TR_PEER_NEED_REQ
+    TR_PEER_ERROR
 }
 PeerEventType;
 
@@ -66,12 +80,12 @@ typedef struct
     float            progress;     /* for PEER_PROGRESS */
     int              err;          /* errno for GOT_ERROR */
     tr_bool          wasPieceData; /* for GOT_DATA */
-    tr_bool          uploadOnly;   /* for UPLOAD_ONLY */
+    tr_port          port;         /* for GOT_PORT */
 }
 tr_peer_event;
 
 #ifdef WIN32
- #define EMSGSIZE WSAEMSGSIZE 
+ #define EMSGSIZE WSAEMSGSIZE
 #endif
 
 /** @} */
