@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.h 10031 2010-01-28 13:33:40Z charles $
+ * $Id: peer-mgr.h 10751 2010-06-14 12:01:50Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -25,6 +25,7 @@
 
 #include "bitfield.h"
 #include "bitset.h"
+#include "history.h"
 #include "net.h"
 #include "peer-common.h" /* struct peer_request */
 #include "publish.h" /* tr_publisher_tag */
@@ -110,6 +111,18 @@ typedef struct tr_peer
 
     time_t                   chokeChangedAt;
 
+    time_t                   lastBlocksAtTime;
+    int                      blocksAt[60];
+
+    time_t                   lastCancelTime;
+    int                      cancelAt[60];
+
+    tr_recentHistory       * blocksSentToClient;
+    tr_recentHistory       * blocksSentToPeer;
+
+    tr_recentHistory       * cancelsSentToClient;
+    tr_recentHistory       * cancelsSentToPeer;
+
     struct tr_peermsgs     * msgs;
     tr_publisher_tag         msgsTag;
 }
@@ -159,9 +172,15 @@ tr_pex * tr_peerMgrArrayToPex( const void * array,
                                size_t       arrayLen,
                                size_t      * setme_pex_count );
 
+/**
+ * @param seedProbability [0..100] for likelihood that the peer is a seed; -1 for unknown
+ */
 void tr_peerMgrAddPex( tr_torrent     * tor,
                        uint8_t          from,
-                       const tr_pex   * pex );
+                       const tr_pex   * pex,
+                       int8_t           seedProbability );
+
+void tr_peerMgrMarkAllAsSeeds( tr_torrent * tor );
 
 void tr_peerMgrSetBlame( tr_torrent        * tor,
                          tr_piece_index_t    pieceIndex,
@@ -194,6 +213,8 @@ void tr_peerMgrTorrentAvailability( const tr_torrent * tor,
 
 struct tr_bitfield* tr_peerMgrGetAvailable( const tr_torrent * tor );
 
+void tr_peerMgrOnBlocklistChanged( tr_peerMgr * manager );
+
 void tr_peerMgrTorrentStats( tr_torrent * tor,
                              int * setmePeersKnown,
                              int * setmePeersConnected,
