@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: publish.c 6795 2008-09-23 19:11:04Z charles $
+ * $Id: publish.c 9868 2010-01-04 21:00:47Z charles $
  */
 
 #include <assert.h>
@@ -21,32 +21,18 @@ struct tr_publisher_node
     void *              user_data;
 };
 
-struct tr_publisher_s
-{
-    tr_list *  list;
-};
-
-tr_publisher_t*
-tr_publisherNew( void )
-{
-    return tr_new0( tr_publisher_t, 1 );
-}
+const tr_publisher TR_PUBLISHER_INIT = { NULL };
 
 void
-tr_publisherFree( tr_publisher_t ** p )
+tr_publisherDestruct( tr_publisher * p )
 {
-    assert( p );
-    assert( *p );
-
-    tr_list_free( &( *p )->list, NULL );
-    tr_free( *p );
-    *p = NULL;
+    tr_list_free( &p->list, NULL );
 }
 
 tr_publisher_tag
-tr_publisherSubscribe( tr_publisher_t * p,
-                       tr_delivery_func func,
-                       void *           user_data )
+tr_publisherSubscribe( tr_publisher     * p,
+                       tr_delivery_func   func,
+                       void *             user_data )
 {
     struct tr_publisher_node * node = tr_new( struct tr_publisher_node, 1 );
 
@@ -57,7 +43,7 @@ tr_publisherSubscribe( tr_publisher_t * p,
 }
 
 void
-tr_publisherUnsubscribe( tr_publisher_t * p,
+tr_publisherUnsubscribe( tr_publisher * p,
                          tr_publisher_tag tag )
 {
     tr_list_remove_data( &p->list, tag );
@@ -65,7 +51,7 @@ tr_publisherUnsubscribe( tr_publisher_t * p,
 }
 
 void
-tr_publisherPublish( tr_publisher_t * p,
+tr_publisherPublish( tr_publisher * p,
                      void *           source,
                      void *           event )
 {
