@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpc-server.c 8685 2009-06-14 01:00:36Z charles $
+ * $Id: rpc-server.c 10985 2010-07-09 15:22:23Z charles $
  */
 
 #include <assert.h>
@@ -31,6 +31,7 @@
 #include "bencode.h"
 #include "crypto.h"
 #include "list.h"
+#include "net.h"
 #include "platform.h"
 #include "ptrarray.h"
 #include "rpcimpl.h"
@@ -38,7 +39,6 @@
 #include "trevent.h"
 #include "utils.h"
 #include "web.h"
-#include "net.h"
 
 /* session-id is used to make cross-site request forgery attacks difficult.
  * Don't disable this feature unless you really know what you're doing!
@@ -73,6 +73,7 @@ struct tr_rpc_server
     time_t             sessionIdExpiresAt;
 
 #ifdef HAVE_ZLIB
+    tr_bool            isStreamInitialized;
     z_stream           stream;
 #endif
 };
@@ -91,7 +92,7 @@ struct tr_rpc_server
 static char*
 get_current_session_id( struct tr_rpc_server * server )
 {
-    const time_t now = time( NULL );
+    const time_t now = tr_time( );
 
     if( !server->sessionId || ( now >= server->sessionIdExpiresAt ) )
     {
@@ -285,7 +286,7 @@ mimetype_guess( const char * path )
         const char *    suffix;
         const char *    mime_type;
     } types[] = {
-        /* these are just the ones we need for serving clutch... */
+        /* these are the ones we need for serving the web client's files... */
         { "css",  "text/css"                  },
         { "gif",  "image/gif"                 },
         { "html", "text/html"                 },
@@ -327,6 +328,15 @@ add_response( struct evhttp_request * req,
         /* FIXME(libevent2): this won't compile under libevent2.
            but we can use evbuffer_reserve_space() + evbuffer_commit_space() instead */
 
+        if( !server->isStreamInitialized )
+        {
+            server->isStreamInitialized = TRUE;
+            server->stream.zalloc = (alloc_func) Z_NULL;
+            server->stream.zfree = (free_func) Z_NULL;
+            server->stream.opaque = (voidpf) Z_NULL;
+            deflateInit( &server->stream, Z_BEST_COMPRESSION );
+        }
+
         server->stream.next_in = (Bytef*) content;
         server->stream.avail_in = content_len;
 
@@ -412,7 +422,7 @@ serve_file( struct evhttp_request * req,
         else
         {
             struct evbuffer * out;
-            const time_t now = time( NULL );
+            const time_t now = tr_time( );
 
             errno = error;
             out = evbuffer_new( );
@@ -429,14 +439,14 @@ serve_file( struct evhttp_request * req,
 }
 
 static void
-handle_clutch( struct evhttp_request * req,
-               struct tr_rpc_server *  server )
+handle_web_client( struct evhttp_request * req,
+                   struct tr_rpc_server *  server )
 {
-    const char * clutchDir = tr_getClutchDir( server->session );
+    const char * webClientDir = tr_getWebClientDir( server->session );
 
     assert( !strncmp( req->uri, "/transmission/web/", 18 ) );
 
-    if( !clutchDir || !*clutchDir )
+    if( !webClientDir || !*webClientDir )
     {
         send_simple_response( req, HTTP_NOTFOUND,
             "<p>Couldn't find Transmission's web interface files!</p>"
@@ -464,7 +474,7 @@ handle_clutch( struct evhttp_request * req,
         else
         {
             char * filename = tr_strdup_printf( "%s%s%s",
-                                 clutchDir,
+                                 webClientDir,
                                  TR_PATH_DELIMITER_STR,
                                  subpath && *subpath ? subpath : "index.html" );
             serve_file( req, server, filename );
@@ -509,7 +519,7 @@ handle_rpc( struct evhttp_request * req,
 
     data->req = req;
     data->server = server;
-    
+
     if( req->type == EVHTTP_REQ_GET )
     {
         const char * q;
@@ -598,17 +608,13 @@ handle_request( struct evhttp_request * req, void * arg )
                || !strcmp( req->uri, "/transmission/clutch" )
                || !strcmp( req->uri, "/" ) )
         {
-            const char * protocol = "http";
-            const char * host = evhttp_find_header( req->input_headers, "Host" );
-            const char * uri = "transmission/web/";
-            char * location = tr_strdup_printf( "%s://%s/%s", protocol, host, uri );
+            const char * location = "transmission/web/";
             evhttp_add_header( req->output_headers, "Location", location );
             send_simple_response( req, HTTP_MOVEPERM, NULL );
-            tr_free( location );
         }
         else if( !strncmp( req->uri, "/transmission/web/", 18 ) )
         {
-            handle_clutch( req, server );
+            handle_web_client( req, server );
         }
         else if( !strncmp( req->uri, "/transmission/upload", 20 ) )
         {
@@ -619,17 +625,16 @@ handle_request( struct evhttp_request * req, void * arg )
         {
             const char * sessionId = get_current_session_id( server );
             char * tmp = tr_strdup_printf(
-                "<p>Please add this header to your HTTP requests:</p>"
-                "<p style=\"padding-left: 20pt;\"><code>%s: %s</code></p>"
-                "<p><b>RPC Application Developers:</b></p>"
-                "<p style=\"padding-left: 20pt;\">As of Transmission 1.53 and 1.61, RPC clients "
-                "need to look for this 409 response containing the phrase \"invalid session-id\".  "
-                "It occurs when the request's "TR_RPC_SESSION_ID_HEADER" header was missing "
-                "(such as during bootstrapping) or expired. "
-                "Either way, you can parse this response's headers for the new session-id.</p>"
-                "<p style=\"padding-left: 20pt;\">This requirement has been added to make "
-                "<a href=\"http://en.wikipedia.org/wiki/Cross-site_request_forgery\">CSRF</a>"
-                " attacks more difficult.</p>",
+                "<p>Your request had an invalid session-id header.</p>"
+                "<p>To fix this, follow these steps:"
+                "<ol><li> When reading a response, get its X-Transmission-Session-Id header and remember it"
+                "<li> Add the updated header to your outgoing requests"
+                "<li> When you get this 409 error message, resend your request with the updated header"
+                "</ol></p>"
+                "<p>This requirement has been added to help prevent "
+                "<a href=\"http://en.wikipedia.org/wiki/Cross-site_request_forgery\">CSRF</a> "
+                "attacks.</p>"
+                "<p><code>%s: %s</code></p>",
                 TR_RPC_SESSION_ID_HEADER, sessionId );
             evhttp_add_header( req->output_headers, TR_RPC_SESSION_ID_HEADER, sessionId );
             send_simple_response( req, 409, tmp );
@@ -719,6 +724,8 @@ void
 tr_rpcSetPort( tr_rpc_server * server,
                tr_port         port )
 {
+    assert( server != NULL );
+
     if( server->port != port )
     {
         server->port = port;
@@ -759,7 +766,7 @@ tr_rpcSetWhitelist( tr_rpc_server * server,
             tr_ninf( MY_NAME, "Adding address to whitelist: %s (And it has a '+' or '-'!  Are you using an old ACL by mistake?)", token );
         else
             tr_ninf( MY_NAME, "Adding address to whitelist: %s", token );
-        
+
         if( walk[len]=='\0' )
             break;
         walk += len + 1;
@@ -859,7 +866,8 @@ closeServer( void * vserver )
     while(( tmp = tr_list_pop_front( &s->whitelist )))
         tr_free( tmp );
 #ifdef HAVE_ZLIB
-    deflateEnd( &s->stream );
+    if( s->isStreamInitialized )
+        deflateEnd( &s->stream );
 #endif
     tr_free( s->sessionId );
     tr_free( s->whitelistStr );
@@ -876,8 +884,7 @@ tr_rpcClose( tr_rpc_server ** ps )
 }
 
 tr_rpc_server *
-tr_rpcInit( tr_session  * session,
-            tr_benc * settings )
+tr_rpcInit( tr_session  * session, tr_benc * settings )
 {
     tr_rpc_server * s;
     tr_bool found;
@@ -899,11 +906,11 @@ tr_rpcInit( tr_session  * session,
 
     found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_WHITELIST_ENABLED, &boolVal );
     assert( found );
-    s->isWhitelistEnabled = boolVal;
+    tr_rpcSetWhitelistEnabled( s, boolVal );
 
     found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, &boolVal );
     assert( found );
-    s->isPasswordEnabled = boolVal;
+    tr_rpcSetPasswordEnabled( s, boolVal );
 
     found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_WHITELIST, &str );
     assert( found );
@@ -911,14 +918,11 @@ tr_rpcInit( tr_session  * session,
 
     found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_USERNAME, &str );
     assert( found );
-    s->username = tr_strdup( str );
+    tr_rpcSetUsername( s, str );
 
     found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_PASSWORD, &str );
     assert( found );
-    if( *str != '{' )
-        s->password = tr_ssha1( str );
-    else
-        s->password = strdup( str );
+    tr_rpcSetPassword( s, str );
 
     found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, &str );
     assert( found );
@@ -932,23 +936,16 @@ tr_rpcInit( tr_session  * session,
     }
     s->bindAddress = address.addr.addr4;
 
-#ifdef HAVE_ZLIB
-    s->stream.zalloc = (alloc_func) Z_NULL;
-    s->stream.zfree = (free_func) Z_NULL;
-    s->stream.opaque = (voidpf) Z_NULL;
-    deflateInit( &s->stream, Z_BEST_COMPRESSION );
-#endif
-
     if( s->isEnabled )
     {
         tr_ninf( MY_NAME, _( "Serving RPC and Web requests on port %d" ), (int) s->port );
         tr_runInEventThread( session, startServer, s );
 
         if( s->isWhitelistEnabled )
-            tr_ninf( MY_NAME, _( "Whitelist enabled" ) );
+            tr_ninf( MY_NAME, "%s", _( "Whitelist enabled" ) );
 
         if( s->isPasswordEnabled )
-            tr_ninf( MY_NAME, _( "Password required" ) );
+            tr_ninf( MY_NAME, "%s", _( "Password required" ) );
     }
 
     return s;
