@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: session.c 10602 2010-05-01 14:19:45Z charles $
+ * $Id: session.c 11005 2010-07-13 13:35:58Z charles $
  */
 
 #include <assert.h>
@@ -42,6 +42,7 @@
 #include "stats.h"
 #include "torrent.h"
 #include "tr-dht.h"
+#include "tr-lpd.h"
 #include "trevent.h"
 #include "utils.h"
 #include "verify.h"
@@ -244,6 +245,7 @@ tr_sessionGetDefaultSettings( const char * configDir UNUSED, tr_benc * d )
     tr_bencDictReserve( d, 35 );
     tr_bencDictAddBool( d, TR_PREFS_KEY_BLOCKLIST_ENABLED,        FALSE );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DHT_ENABLED,              TRUE );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_LPD_ENABLED,              FALSE );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_DOWNLOAD_DIR,             tr_getDefaultDownloadDir( ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_DSPEED,                   100 );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED,           FALSE );
@@ -281,6 +283,8 @@ tr_sessionGetDefaultSettings( const char * configDir UNUSED, tr_benc * d )
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_WHITELIST,            TR_DEFAULT_RPC_WHITELIST );
     tr_bencDictAddBool( d, TR_PREFS_KEY_RPC_WHITELIST_ENABLED,    TRUE );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_RPC_PORT,                 atoi( TR_DEFAULT_RPC_PORT_STR ) );
+    tr_bencDictAddStr ( d, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, "" );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, FALSE );
     tr_bencDictAddBool( d, TR_PREFS_KEY_ALT_SPEED_ENABLED,        FALSE );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_ALT_SPEED_UP,             50 ); /* half the regular */
     tr_bencDictAddInt ( d, TR_PREFS_KEY_ALT_SPEED_DOWN,           50 ); /* half the regular */
@@ -294,6 +298,8 @@ tr_sessionGetDefaultSettings( const char * configDir UNUSED, tr_benc * d )
     tr_bencDictAddInt ( d, TR_PREFS_KEY_UPLOAD_SLOTS_PER_TORRENT, 14 );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_BIND_ADDRESS_IPV4,        TR_DEFAULT_BIND_ADDRESS_IPV4 );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_BIND_ADDRESS_IPV6,        TR_DEFAULT_BIND_ADDRESS_IPV6 );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_START,                    TRUE );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_TRASH_ORIGINAL,           FALSE );
 }
 
 void
@@ -304,6 +310,7 @@ tr_sessionGetSettings( tr_session * s, struct tr_benc * d )
     tr_bencDictReserve( d, 30 );
     tr_bencDictAddBool( d, TR_PREFS_KEY_BLOCKLIST_ENABLED,        tr_blocklistIsEnabled( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DHT_ENABLED,              s->isDHTEnabled );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_LPD_ENABLED,              s->isLPDEnabled );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_DOWNLOAD_DIR,             s->downloadDir );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_DSPEED,                   tr_sessionGetSpeedLimit( s, TR_DOWN ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED,           tr_sessionIsSpeedLimited( s, TR_DOWN ) );
@@ -320,6 +327,8 @@ tr_sessionGetSettings( tr_session * s, struct tr_benc * d )
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_PORT_RANDOM_LOW,     s->randomPortLow );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_PORT_RANDOM_HIGH,    s->randomPortHigh );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_SOCKET_TOS,          s->peerSocketTOS );
+    if(s->peer_congestion_algorithm && s->peer_congestion_algorithm[0])
+        tr_bencDictAddStr ( d, TR_PREFS_KEY_PEER_CONGESTION_ALGORITHM, s->peer_congestion_algorithm );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PEX_ENABLED,              s->isPexEnabled );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PORT_FORWARDING,          tr_sessionIsPortForwardingEnabled( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PREALLOCATION,            s->preallocationMode );
@@ -341,6 +350,8 @@ tr_sessionGetSettings( tr_session * s, struct tr_benc * d )
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_USERNAME,             tr_sessionGetRPCUsername( s ) );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_WHITELIST,            tr_sessionGetRPCWhitelist( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_RPC_WHITELIST_ENABLED,    tr_sessionGetRPCWhitelistEnabled( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, tr_sessionIsTorrentDoneScriptEnabled( s ) );
+    tr_bencDictAddStr ( d, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, tr_sessionGetTorrentDoneScript( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_ALT_SPEED_ENABLED,        tr_sessionUsesAltSpeed( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_ALT_SPEED_UP,             tr_sessionGetAltSpeed( s, TR_UP ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_ALT_SPEED_DOWN,           tr_sessionGetAltSpeed( s, TR_DOWN ) );
@@ -354,6 +365,8 @@ tr_sessionGetSettings( tr_session * s, struct tr_benc * d )
     tr_bencDictAddInt ( d, TR_PREFS_KEY_UPLOAD_SLOTS_PER_TORRENT, s->uploadSlotsPerTorrent );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_BIND_ADDRESS_IPV4,        tr_ntop_non_ts( &s->public_ipv4->addr ) );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_BIND_ADDRESS_IPV6,        tr_ntop_non_ts( &s->public_ipv6->addr ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_START,                    !tr_sessionGetPaused( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_TRASH_ORIGINAL,           tr_sessionGetDeleteSource( s ) );
 }
 
 tr_bool
@@ -619,6 +632,11 @@ tr_sessionInitImpl( void * vdata )
         tr_dhtInit( session, &session->public_ipv4->addr );
     }
 
+    if( !session->isLPDEnabled )
+        tr_ndbg( "LPD", _( "Local Peer Discovery disabled" ) );
+    else if( tr_lpdInit( session, &session->public_ipv4->addr ) )
+        tr_ninf( "LPD", _( "Local Peer Discovery active" ) );
+
     /* cleanup */
     tr_bencFree( &settings );
     data->done = TRUE;
@@ -661,12 +679,20 @@ sessionSetImpl( void * vdata )
         tr_sessionSetPexEnabled( session, boolVal );
     if( tr_bencDictFindBool( settings, TR_PREFS_KEY_DHT_ENABLED, &boolVal ) )
         tr_sessionSetDHTEnabled( session, boolVal );
+    if( tr_bencDictFindBool( settings, TR_PREFS_KEY_LPD_ENABLED, &boolVal ) )
+        tr_sessionSetLPDEnabled( session, boolVal );
     if( tr_bencDictFindInt( settings, TR_PREFS_KEY_ENCRYPTION, &i ) )
         tr_sessionSetEncryption( session, i );
     if( tr_bencDictFindInt( settings, TR_PREFS_KEY_PEER_SOCKET_TOS, &i ) )
         session->peerSocketTOS = i;
+    if( tr_bencDictFindStr( settings, TR_PREFS_KEY_PEER_CONGESTION_ALGORITHM, &str ) )
+        session->peer_congestion_algorithm = tr_strdup(str);
     if( tr_bencDictFindBool( settings, TR_PREFS_KEY_BLOCKLIST_ENABLED, &boolVal ) )
         tr_blocklistSetEnabled( session, boolVal );
+    if( tr_bencDictFindBool( settings, TR_PREFS_KEY_START, &boolVal ) )
+        tr_sessionSetPaused( session, !boolVal );
+    if( tr_bencDictFindBool( settings, TR_PREFS_KEY_TRASH_ORIGINAL, &boolVal) )
+        tr_sessionSetDeleteSource( session, boolVal );
 
     /* files and directories */
     if( tr_bencDictFindInt( settings, TR_PREFS_KEY_PREALLOCATION, &i ) )
@@ -780,6 +806,15 @@ sessionSetImpl( void * vdata )
         turtle->isEnabled = boolVal;
     turtleBootstrap( session, turtle );
 
+    /**
+    ***  Scripts
+    **/
+
+    if( tr_bencDictFindBool( settings, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, &boolVal ) )
+        tr_sessionSetTorrentDoneScriptEnabled( session, boolVal );
+    if( tr_bencDictFindStr( settings, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, &str ) )
+        tr_sessionSetTorrentDoneScript( session, str );
+
     data->done = TRUE;
 }
 
@@ -964,9 +999,7 @@ setPeerPort( tr_session * session, tr_port port )
 void
 tr_sessionSetPeerPort( tr_session * session, tr_port port )
 {
-    assert( tr_isSession( session ) );
-
-    if( session->peerPort != port )
+    if( tr_isSession( session ) && ( session->peerPort != port ) )
     {
         setPeerPort( session, port );
     }
@@ -975,9 +1008,7 @@ tr_sessionSetPeerPort( tr_session * session, tr_port port )
 tr_port
 tr_sessionGetPeerPort( const tr_session * session )
 {
-    assert( tr_isSession( session ) );
-
-    return session->peerPort;
+    return tr_isSession( session ) ? session->peerPort : 0;
 }
 
 tr_port
@@ -1018,22 +1049,12 @@ tr_sessionGetPortForwarding( const tr_session * session )
 ****
 ***/
 
-static void
-updateSeedRatio( tr_session * session )
-{
-    tr_torrent * tor = NULL;
-
-    while(( tor = tr_torrentNext( session, tor )))
-        tor->needsSeedRatioCheck = TRUE;
-}
-
 void
 tr_sessionSetRatioLimited( tr_session * session, tr_bool isLimited )
 {
     assert( tr_isSession( session ) );
 
     session->isRatioLimited = isLimited;
-    updateSeedRatio( session );
 }
 
 void
@@ -1042,7 +1063,6 @@ tr_sessionSetRatioLimit( tr_session * session, double desiredRatio )
     assert( tr_isSession( session ) );
 
     session->desiredRatio = desiredRatio;
-    updateSeedRatio( session );
 }
 
 tr_bool
@@ -1213,7 +1233,7 @@ turtleCheckClock( tr_session * s, struct tr_turtle_info * t )
 }
 
 /* Called after the turtle's fields are loaded from an outside source.
- * It initializes the implementation fields 
+ * It initializes the implementation fields
  * and turns on turtle mode if the clock settings say to. */
 static void
 turtleBootstrap( tr_session * session, struct tr_turtle_info * turtle )
@@ -1468,6 +1488,42 @@ tr_sessionGetPeerLimitPerTorrent( const tr_session * session )
 ****
 ***/
 
+void
+tr_sessionSetPaused( tr_session * session, tr_bool isPaused )
+{
+    assert( tr_isSession( session ) );
+
+    session->pauseAddedTorrent = isPaused;
+}
+
+tr_bool
+tr_sessionGetPaused( const tr_session * session )
+{
+    assert( tr_isSession( session ) );
+
+    return session->pauseAddedTorrent;
+}
+
+void
+tr_sessionSetDeleteSource( tr_session * session, tr_bool deleteSource )
+{
+    assert( tr_isSession( session ) );
+
+    session->deleteSourceTorrent = deleteSource;
+}
+
+tr_bool
+tr_sessionGetDeleteSource( const tr_session * session )
+{
+    assert( tr_isSession( session ) );
+
+    return session->deleteSourceTorrent;
+}
+
+/***
+****
+***/
+
 double
 tr_sessionGetPieceSpeed( const tr_session * session, tr_direction dir )
 {
@@ -1514,6 +1570,9 @@ sessionCloseImpl( void * vsession )
 
     free_incoming_peer_port( session );
 
+    if( session->isLPDEnabled )
+        tr_lpdUninit( session );
+
     if( session->isDHTEnabled )
         tr_dhtUninit( session );
 
@@ -1619,6 +1678,7 @@ tr_sessionClose( tr_session * session )
         tr_bencFree( session->metainfoLookup );
         tr_free( session->metainfoLookup );
     }
+    tr_free( session->torrentDoneScript );
     tr_free( session->buffer );
     tr_free( session->tag );
     tr_free( session->configDir );
@@ -1629,6 +1689,7 @@ tr_sessionClose( tr_session * session )
     tr_free( session->proxy );
     tr_free( session->proxyUsername );
     tr_free( session->proxyPassword );
+    tr_free( session->peer_congestion_algorithm );
     tr_free( session );
 }
 
@@ -1655,8 +1716,7 @@ tr_sessionLoadTorrents( tr_session * session,
         struct dirent *d;
         for( d = readdir( odir ); d != NULL; d = readdir( odir ) )
         {
-            if( d->d_name && d->d_name[0] != '.' ) /* skip dotfiles, ., and ..
-                                                     */
+            if( tr_str_has_suffix( d->d_name, ".torrent" ) )
             {
                 tr_torrent * tor;
                 char * path = tr_buildPath( dirname, d->d_name, NULL );
@@ -1747,6 +1807,29 @@ tr_sessionSetDHTEnabled( tr_session * session, tr_bool enabled )
         tr_runInEventThread( session, toggleDHTImpl, session );
 }
 
+void
+tr_sessionSetLPDEnabled( tr_session * session,
+                         tr_bool      enabled )
+{
+    assert( tr_isSession( session ) );
+
+    session->isLPDEnabled = ( enabled != 0 );
+}
+
+tr_bool
+tr_sessionIsLPDEnabled( const tr_session * session )
+{
+    assert( tr_isSession( session ) );
+
+    return session->isLPDEnabled;
+}
+
+tr_bool
+tr_sessionAllowsLPD( const tr_session * session )
+{
+    return tr_sessionIsLPDEnabled( session );
+}
+
 /***
 ****
 ***/
@@ -1902,6 +1985,8 @@ tr_sessionReloadBlocklists( tr_session * session )
 {
     closeBlocklists( session );
     loadBlocklists( session );
+
+    tr_peerMgrOnBlocklistChanged( session->peerMgr );
 }
 
 int
@@ -2015,7 +2100,7 @@ metainfoLookupInit( tr_session * session )
         struct dirent *d;
         while(( d = readdir( odir )))
         {
-            if( d->d_name && d->d_name[0] != '.' )
+            if( tr_str_has_suffix( d->d_name, ".torrent" ) )
             {
                 tr_info inf;
                 char * path = tr_buildPath( dirname, d->d_name, NULL );
@@ -2358,17 +2443,43 @@ tr_sessionSetProxyPassword( tr_session * session,
     }
 }
 
-int
-tr_sessionGetActiveTorrentCount( tr_session * session )
-{
-    int ret = 0;
-    tr_torrent * tor = NULL;
+/****
+*****
+****/
 
+tr_bool
+tr_sessionIsTorrentDoneScriptEnabled( const tr_session * session )
+{
     assert( tr_isSession( session ) );
 
-    while(( tor = tr_torrentNext( session, tor )))
-        if( tr_torrentGetActivity( tor ) != TR_STATUS_STOPPED )
-            ++ret;
+    return session->isTorrentDoneScriptEnabled;
+}
 
-    return ret;
+void
+tr_sessionSetTorrentDoneScriptEnabled( tr_session * session, tr_bool isEnabled )
+{
+    assert( tr_isSession( session ) );
+    assert( tr_isBool( isEnabled ) );
+
+    session->isTorrentDoneScriptEnabled = isEnabled;
+}
+
+const char *
+tr_sessionGetTorrentDoneScript( const tr_session * session )
+{
+    assert( tr_isSession( session ) );
+
+    return session->torrentDoneScript;
+}
+
+void
+tr_sessionSetTorrentDoneScript( tr_session * session, const char * scriptFilename )
+{
+    assert( tr_isSession( session ) );
+
+    if( session->torrentDoneScript != scriptFilename )
+    {
+        tr_free( session->torrentDoneScript );
+        session->torrentDoneScript = tr_strdup( scriptFilename );
+    }
 }
