@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: session.h 10593 2010-05-01 13:40:42Z charles $
+ * $Id: session.h 10638 2010-05-08 22:42:28Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -84,14 +84,18 @@ struct tr_session
     tr_bool                      isPortRandom;
     tr_bool                      isPexEnabled;
     tr_bool                      isDHTEnabled;
+    tr_bool                      isLPDEnabled;
     tr_bool                      isBlocklistEnabled;
     tr_bool                      isProxyEnabled;
     tr_bool                      isProxyAuthEnabled;
+    tr_bool                      isTorrentDoneScriptEnabled;
     tr_bool                      isClosed;
     tr_bool                      useLazyBitfield;
     tr_bool                      isIncompleteFileNamingEnabled;
     tr_bool                      isRatioLimited;
     tr_bool                      isIncompleteDirEnabled;
+    tr_bool                      pauseAddedTorrent;
+    tr_bool                      deleteSourceTorrent;
 
     tr_benc                      removedTorrents;
 
@@ -122,10 +126,13 @@ struct tr_session
 
     int                          proxyPort;
     int                          peerSocketTOS;
+    char *                       peer_congestion_algorithm;
 
     int                          torrentCount;
     tr_torrent *                 torrentList;
 
+    char *                       torrentDoneScript;
+
     char *                       tag;
     char *                       configDir;
     char *                       downloadDir;
@@ -176,6 +183,8 @@ struct tr_session
 
 tr_bool      tr_sessionAllowsDHT( const tr_session * session );
 
+tr_bool      tr_sessionAllowsLPD( const tr_session * session );
+
 const char * tr_sessionFindTorrentFile( const tr_session * session,
                                         const char *       hashString );
 
@@ -196,6 +205,8 @@ const struct tr_address*  tr_sessionGetPublicAddress( const tr_session *, int tr
 
 struct tr_bindsockets * tr_sessionGetBindSockets( tr_session * );
 
+int tr_sessionCountTorrents( const tr_session * session ); 
+
 enum
 {
     SESSION_MAGIC_NUMBER = 3845,
