@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.c 10305 2010-03-06 19:33:04Z charles $
+ * $Id: utils.c 10964 2010-07-07 16:48:24Z charles $
  */
 
 #ifdef HAVE_MEMMEM
@@ -16,6 +16,7 @@
 
 #if defined(SYS_DARWIN)
  #define HAVE_GETPAGESIZE
+ #define HAVE_ICONV_OPEN
  #define HAVE_VALLOC
  #undef HAVE_POSIX_MEMALIGN /* not supported on OS X 10.5 and lower */
 #endif
@@ -28,12 +29,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> /* strerror, memset, memmem */
+#include <time.h> /* nanosleep() */
 
+#ifdef HAVE_ICONV_OPEN
+ #include <iconv.h>
+#endif
 #include <libgen.h> /* basename */
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h> /* usleep, stat, getcwd, getpagesize */
+#include <unistd.h> /* stat(), getcwd(), getpagesize() */
 
 #include "event.h"
 
@@ -52,6 +57,8 @@
 #include "version.h"
 
 
+time_t transmission_now = 0;
+
 int                   messageLevel = TR_MSG_INF;
 static tr_lock *      messageLock = NULL;
 static tr_bool        messageQueuing = FALSE;
@@ -69,18 +76,6 @@ static int            messageQueueCount = 0;
 ****
 ***/
 
-time_t transmission_now = 0;
-
-void
-tr_timeUpdate( time_t now )
-{
-    transmission_now = now;
-}
-
-/***
-****
-***/
-
 void
 tr_msgInit( void )
 {
@@ -783,6 +778,25 @@ tr_strstrip( char * str )
     return str;
 }
 
+tr_bool
+tr_str_has_suffix( const char *str, const char *suffix )
+{
+    size_t str_len;
+    size_t suffix_len;
+
+    if( !str )
+        return FALSE;
+    if( !suffix )
+        return TRUE;
+
+    str_len = strlen( str );
+    suffix_len = strlen( suffix );
+    if( str_len < suffix_len )
+        return FALSE;
+
+    return !strncasecmp( str + str_len - suffix_len, suffix, suffix_len );
+}
+
 /****
 *****
 ****/
@@ -797,12 +811,15 @@ tr_date( void )
 }
 
 void
-tr_wait_msec( uint64_t delay_milliseconds )
+tr_wait_msec( long int msec )
 {
 #ifdef WIN32
-    Sleep( (DWORD)delay_milliseconds );
+    Sleep( (DWORD)msec );
 #else
-    usleep( 1000 * delay_milliseconds );
+    struct timespec ts;
+    ts.tv_sec = msec / 1000;
+    ts.tv_nsec = ( msec % 1000 ) * 1000000;
+    nanosleep( &ts, NULL );
 #endif
 }
 
@@ -949,7 +966,7 @@ tr_urlIsValidTracker( const char * url )
     tr_bool valid;
     char * scheme = NULL;
 
-    valid = isValidURLChars( url ) 
+    valid = isValidURLChars( url )
          && !tr_urlParse( url, -1, &scheme, NULL, NULL, NULL )
          && ( scheme != NULL )
          && ( !strcmp(scheme,"http") || !strcmp(scheme,"https") );
@@ -965,7 +982,7 @@ tr_urlIsValid( const char * url )
     tr_bool valid;
     char * scheme = NULL;
 
-    valid = isValidURLChars( url ) 
+    valid = isValidURLChars( url )
          && !tr_urlParse( url, -1, &scheme, NULL, NULL, NULL )
          && ( scheme != NULL )
          && ( !strcmp(scheme,"http") || !strcmp(scheme,"https") || !strcmp(scheme,"ftp") || !strcmp(scheme,"sftp") );
@@ -1166,8 +1183,68 @@ tr_lowerBound( const void * key,
 ****
 ***/
 
+static char*
+strip_non_utf8( const char * in, size_t inlen )
+{
+    char * ret;
+    const char * end;
+    const char zero = '\0';
+    struct evbuffer * buf = evbuffer_new( );
+ 
+    while( !tr_utf8_validate( in, inlen, &end ) )
+    {
+        const int good_len = end - in;
+
+        evbuffer_add( buf, in, good_len );
+        inlen -= ( good_len + 1 );
+        in += ( good_len + 1 );
+        evbuffer_add( buf, "?", 1 );
+    }
+ 
+    evbuffer_add( buf, in, inlen );
+    evbuffer_add( buf, &zero, 1 );
+    ret = tr_memdup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
+    evbuffer_free( buf );
+    return ret;
+}
+
+static char*
+to_utf8( const char * in, size_t inlen )
+{
+    char * ret = NULL;
+
+#ifdef HAVE_ICONV_OPEN
+    int i;
+    const char * encodings[] = { "CURRENT", "ISO-8859-15" };
+    const int encoding_count = sizeof(encodings) / sizeof(encodings[1]);
+    const size_t buflen = inlen*4 + 10;
+    char * out = tr_new( char, buflen );
+
+    for( i=0; !ret && i<encoding_count; ++i )
+    {
+        char * inbuf = (char*) in;
+        char * outbuf = out;
+        size_t inbytesleft = inlen;
+        size_t outbytesleft = buflen;
+        const char * test_encoding = encodings[i];
+
+        iconv_t cd = iconv_open( "UTF-8", test_encoding );
+        if( cd != (iconv_t)-1 ) {
+            if( iconv( cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft ) != (size_t)-1 )
+                ret = tr_strndup( out, buflen-outbytesleft );
+            iconv_close( cd );
+        }
+    }
+#endif
+
+    if( ret == NULL )
+        ret = strip_non_utf8( in, inlen );
+
+    return ret;
+}
+
 char*
-tr_utf8clean( const char * str, int max_len, tr_bool * err )
+tr_utf8clean( const char * str, int max_len )
 {
     char * ret;
     const char * end;
@@ -1175,36 +1252,10 @@ tr_utf8clean( const char * str, int max_len, tr_bool * err )
     if( max_len < 0 )
         max_len = (int) strlen( str );
 
-    if( err != NULL )
-        *err = FALSE;
-
     if( tr_utf8_validate( str, max_len, &end  ) )
-    {
         ret = tr_strndup( str, max_len );
-    }
     else
-    {
-        const char zero = '\0';
-        struct evbuffer * buf = evbuffer_new( );
-
-        while( !tr_utf8_validate ( str, max_len, &end ) )
-        {
-            const int good_len = end - str;
-
-            evbuffer_add( buf, str, good_len );
-            max_len -= ( good_len + 1 );
-            str += ( good_len + 1 );
-            evbuffer_add( buf, "?", 1 );
-
-            if( err != NULL )
-                *err = TRUE;
-        }
-
-        evbuffer_add( buf, str, max_len );
-        evbuffer_add( buf, &zero, 1 );
-        ret = tr_memdup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
-        evbuffer_free( buf );
-    }
+        ret = to_utf8( str, max_len );
 
     assert( tr_utf8_validate( ret, -1, NULL ) );
     return ret;
@@ -1352,9 +1403,8 @@ tr_parseNumberRange( const char * str_in, int len, int * setmeCount )
 double
 tr_truncd( double x, int decimal_places )
 {
-    static const int multiplier[] = { 1, 10, 100, 1000, 10000, 100000, 1000000, 10000000 };
-    const int64_t i = multiplier[decimal_places];
-    double x2 = (int64_t)(x*i);
+    const int i = (int) pow( 10, decimal_places );  
+    double x2 = (int)(x*i);
     return x2 / i;
 }
 
@@ -1386,7 +1436,7 @@ tr_moveFile( const char * oldpath, const char * newpath, tr_bool * renamed )
     char * buf;
     struct stat st;
     off_t bytesLeft;
-    off_t buflen;
+    const off_t buflen = 1024 * 128; /* 128 KiB buffer */
 
     /* make sure the old file exists */
     if( stat( oldpath, &st ) ) {
@@ -1421,7 +1471,6 @@ tr_moveFile( const char * oldpath, const char * newpath, tr_bool * renamed )
     /* copy the file */
     in = tr_open_file_for_scanning( oldpath );
     out = tr_open_file_for_writing( newpath );
-    buflen = stat( newpath, &st ) ? 4096 : st.st_blksize;
     buf = tr_valloc( buflen );
     while( bytesLeft > 0 )
     {
