@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: verify.c 10274 2010-02-23 06:06:45Z charles $
+ * $Id: verify.c 10905 2010-06-29 01:21:38Z charles $
  */
 
 #include <unistd.h> /* S_ISREG */
@@ -39,7 +39,7 @@
 
 enum
 {
-    MSEC_TO_SLEEP_PER_SECOND_DURING_VERIFY = 200
+    MSEC_TO_SLEEP_PER_SECOND_DURING_VERIFY = 100
 };
 
 /* #define STOPWATCH */
@@ -60,9 +60,11 @@ verifyTorrent( tr_torrent * tor, tr_bool * stopFlag )
     tr_piece_index_t pieceIndex = 0;
     const time_t begin = tr_time( );
     time_t end;
-    const int64_t buflen = 16384;
+    const int64_t buflen = 1024 * 128; /* 128 KiB buffer */
     uint8_t * buffer = tr_valloc( buflen );
 
+    tr_torrentUncheck( tor );
+
     SHA1_Init( &sha );
 
     while( !*stopFlag && ( pieceIndex < tor->info.pieceCount ) )
@@ -327,22 +329,6 @@ compareVerifyByTorrent( const void * va,
     return a->torrent - b;
 }
 
-tr_bool
-tr_verifyInProgress( const tr_torrent * tor )
-{
-    tr_bool found = FALSE;
-    tr_lock * lock = getVerifyLock( );
-    tr_lockLock( lock );
-
-    assert( tr_isTorrent( tor ) );
-
-    found = ( tor == currentNode.torrent )
-         || ( tr_list_find( verifyList, tor, compareVerifyByTorrent ) != NULL );
-
-    tr_lockUnlock( lock );
-    return found;
-}
-
 void
 tr_verifyRemove( tr_torrent * tor )
 {
