@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: AddWindowController.h 10161 2010-02-11 01:34:32Z livings124 $
+ * $Id: AddWindowController.h 10465 2010-04-12 00:55:31Z livings124 $
  *
  * Copyright (c) 2008-2010 Transmission authors and contributors
  *
@@ -44,7 +44,7 @@
     NSString * fDestination, * fTorrentFile;
     BOOL fLockDestination;
     
-    BOOL fDeleteTorrent, fDeleteEnable;
+    BOOL fDeleteTorrentInitial, fDeleteEnableInitial;
     NSInteger fGroupValue;
     
     NSTimer * fTimer;
