@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: AddWindowController.m 10345 2010-03-10 03:45:15Z livings124 $
+ * $Id: AddWindowController.m 10493 2010-04-17 14:01:55Z livings124 $
  *
  * Copyright (c) 2008-2010 Transmission authors and contributors
  *
@@ -69,8 +69,8 @@
         
         fTorrentFile = [[torrentFile stringByExpandingTildeInPath] retain];
         
-        fDeleteTorrent = deleteTorrent;
-        fDeleteEnable = canToggleDelete;
+        fDeleteTorrentInitial = deleteTorrent;
+        fDeleteEnableInitial = canToggleDelete;
         
         fGroupValue = [torrent groupValue];
         
@@ -92,6 +92,7 @@
     NSString * name = [fTorrent name];
     [[self window] setTitle: name];
     [fNameField setStringValue: name];
+    [fNameField setToolTip: name];
     
     [fIconView setImage: [fTorrent icon]];
     
@@ -112,8 +113,8 @@
     
     [fStartCheck setState: [[NSUserDefaults standardUserDefaults] boolForKey: @"AutoStartDownload"] ? NSOnState : NSOffState];
     
-    [fDeleteCheck setState: fDeleteTorrent ? NSOnState : NSOffState];
-    [fDeleteCheck setEnabled: fDeleteEnable];
+    [fDeleteCheck setState: fDeleteTorrentInitial ? NSOnState : NSOffState];
+    [fDeleteCheck setEnabled: fDeleteEnableInitial];
     
     if (fDestination)
         [self setDestinationPath: fDestination];
@@ -274,16 +275,11 @@
         [fVerifyIndicator setIndeterminate: waiting];
         if (!waiting)
             [fVerifyIndicator setDoubleValue: [fTorrent checkingProgress]];
-        else
-            [fVerifyIndicator startAnimation: self];
         
-        [fVerifyIndicator setHidden: NO];
+        [fVerifyIndicator startAnimation: self];
     }
     else
-    {
         [fVerifyIndicator stopAnimation: self];
-        [fVerifyIndicator setHidden: YES];
-    }
 }
 
 - (void) confirmAdd
