@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: BadgeView.m 7666 2009-01-11 18:04:45Z livings124 $
+ * $Id: BadgeView.m 9844 2010-01-01 21:12:04Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,7 @@
 #import "BadgeView.h"
 #import "NSStringAdditions.h"
 
-#define BETWEEN_PADDING 2.0f
+#define BETWEEN_PADDING 2.0
 
 @interface BadgeView (Private)
 
@@ -72,19 +72,19 @@
 
 - (void) drawRect: (NSRect) rect
 {
-    [[NSImage imageNamed: @"NSApplicationIcon"] drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+    [[NSApp applicationIconImage] drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
     
     if (fQuitting)
     {
         NSImage * quitBadge = [NSImage imageNamed: @"QuitBadge.png"];
         [self badge: quitBadge string: NSLocalizedString(@"Quitting", "Dock Badger -> quit")
-                atHeight: (rect.size.height - [quitBadge size].height) * 0.5f adjustForQuit: YES];
+                atHeight: (NSHeight(rect) - [quitBadge size].height) * 0.5 adjustForQuit: YES];
         return;
     }
     
-    BOOL upload = fUploadRate >= 0.1f,
-        download = fDownloadRate >= 0.1f;
-    CGFloat bottom = 0.0f;
+    const BOOL upload = fUploadRate >= 0.1,
+            download = fDownloadRate >= 0.1;
+    CGFloat bottom = 0.0;
     if (upload)
     {
         NSImage * uploadBadge = [NSImage imageNamed: @"UploadBadge.png"];
@@ -101,34 +101,42 @@
 
 @implementation BadgeView (Private)
 
-//dock icon must have locked focus
 - (void) badge: (NSImage *) badge string: (NSString *) string atHeight: (CGFloat) height adjustForQuit: (BOOL) quit
 {
     if (!fAttributes)
     {
         NSShadow * stringShadow = [[NSShadow alloc] init];
-        [stringShadow setShadowOffset: NSMakeSize(2.0f, -2.0f)];
-        [stringShadow setShadowBlurRadius: 4.0f];
+        [stringShadow setShadowOffset: NSMakeSize(2.0, -2.0)];
+        [stringShadow setShadowBlurRadius: 4.0];
         
-        fAttributes = [[NSDictionary alloc] initWithObjectsAndKeys:
-            [NSColor whiteColor], NSForegroundColorAttributeName,
-            [NSFont boldSystemFontOfSize: 26.0f], NSFontAttributeName, stringShadow, NSShadowAttributeName, nil];
+        fAttributes = [[NSMutableDictionary alloc] initWithCapacity: 3];
+        [fAttributes setObject: [NSColor whiteColor] forKey: NSForegroundColorAttributeName];
+        [fAttributes setObject: stringShadow forKey: NSShadowAttributeName];
         
         [stringShadow release];
     }
     
-    NSRect badgeRect = NSZeroRect;
+    NSRect badgeRect;
     badgeRect.size = [badge size];
+    badgeRect.origin.x = 0.0;
     badgeRect.origin.y = height;
     
-    [badge drawInRect: badgeRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0f];
+    [badge drawInRect: badgeRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+    
+    //make sure text fits on the badge
+    CGFloat fontSize = 26.0;
+    NSSize stringSize;
+    do
+    {
+        [fAttributes setObject: [NSFont boldSystemFontOfSize: fontSize] forKey: NSFontAttributeName];
+        stringSize = [string sizeWithAttributes: fAttributes];
+        fontSize -= 1.0;
+    } while (NSWidth(badgeRect) < stringSize.width);
     
     //string is in center of image
-    NSSize stringSize = [string sizeWithAttributes: fAttributes];
-    
-    NSRect stringRect = badgeRect;
-    stringRect.origin.x += (badgeRect.size.width - stringSize.width) * 0.5f;
-    stringRect.origin.y += (badgeRect.size.height - stringSize.height) * 0.5f + (quit ? 2.0f : 1.0f); //adjust for shadow, extra for quit
+    NSRect stringRect;
+    stringRect.origin.x = NSMidX(badgeRect) - stringSize.width * 0.5;
+    stringRect.origin.y = NSMidY(badgeRect) - stringSize.height * 0.5 + (quit ? 2.0 : 1.0); //adjust for shadow, extra for quit
     stringRect.size = stringSize;
     
     [string drawInRect: stringRect withAttributes: fAttributes];
