@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Controller.h 10073 2010-02-02 01:47:22Z livings124 $
+ * $Id: Controller.h 10465 2010-04-12 00:55:31Z livings124 $
  *
  * Copyright (c) 2005-2010 Transmission authors and contributors
  *
@@ -27,6 +27,7 @@
 #import <Quartz/Quartz.h>
 #import <Growl/Growl.h>
 
+@class AddMagnetWindowController;
 @class AddWindowController;
 @class Badger;
 @class DragOverlayWindow;
@@ -117,13 +118,15 @@ typedef enum
 }
 
 - (void) openFiles: (NSArray *) filenames addType: (addType) type forcePath: (NSString *) path;
-- (void) openMagnet: (NSString *) address;
 
 - (void) askOpenConfirmed: (AddWindowController *) addController add: (BOOL) add;
 - (void) openCreatedFile: (NSNotification *) notification;
 - (void) openFilesWithDict: (NSDictionary *) dictionary;
 - (void) openShowSheet: (id) sender;
 
+- (void) openMagnet: (NSString *) address;
+- (void) askOpenMagnetConfirmed: (AddMagnetWindowController *) addController add: (BOOL) add;
+
 - (void) invalidOpenAlert: (NSString *) filename;
 - (void) invalidOpenMagnetAlert: (NSString *) address;
 - (void) duplicateOpenAlert: (NSString *) name;
