@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: CreatorWindowController.m 10240 2010-02-20 15:57:34Z charles $
+ * $Id: CreatorWindowController.m 10669 2010-05-20 00:16:49Z livings124 $
  *
  * Copyright (c) 2007-2010 Transmission authors and contributors
  *
@@ -23,8 +23,11 @@
  *****************************************************************************/
 
 #import "CreatorWindowController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
-#import "utils.h" //tr_urlIsValidTracker
+
+#import "transmission.h" // required by utils.h
+#import "utils.h" // tr_urlIsValidTracker()
 
 #define TRACKER_ADD_TAG 0
 #define TRACKER_REMOVE_TAG 1
@@ -228,8 +231,8 @@
                     ? NSLocalizedString(@"A transfer marked as private with no tracker addresses will be unable to connect to peers."
                         " The torrent file will only be useful if you plan to upload the file to a tracker website"
                         " that will add the addresses for you.", "Create torrent -> blank address -> message")
-                    : NSLocalizedString(@"The transfer will not contact trackers for peers,"
-                        " and will have to rely solely on DHT and PEX to download and seed.",
+                    : NSLocalizedString(@"The transfer will not contact trackers for peers, and will have to rely solely on"
+                        " non-tracker peer discovery methods such as PEX and DHT to download and seed.",
                         "Create torrent -> blank address -> message");
         
         [alert setInformativeText: infoString];
@@ -321,6 +324,90 @@
     [fTrackerAddRemoveControl setEnabled: [fTrackerTable numberOfSelectedRows] > 0 forSegment: TRACKER_REMOVE_TAG];
 }
 
+- (void) copy: (id) sender
+{
+    NSArray * addresses = [fTrackers objectsAtIndexes: [fTrackerTable selectedRowIndexes]];
+    NSString * text = [addresses componentsJoinedByString: @"\n"];
+    
+    NSPasteboard * pb = [NSPasteboard generalPasteboard];
+    if ([NSApp isOnSnowLeopardOrBetter])
+    {
+        [pb clearContents];
+        [pb writeObjects: [NSArray arrayWithObject: text]];
+    }
+    else
+    {
+        [pb declareTypes: [NSArray arrayWithObject: NSStringPboardType] owner: nil];
+        [pb setString: text forType: NSStringPboardType];
+    }
+}
+
+- (BOOL) validateMenuItem: (NSMenuItem *) menuItem
+{
+    const SEL action = [menuItem action];
+    
+    if (action == @selector(copy:))
+        return [[self window] firstResponder] == fTrackerTable && [fTrackerTable numberOfSelectedRows] > 0;
+    
+    if (action == @selector(paste:))
+        return [[self window] firstResponder] == fTrackerTable
+            && ([NSApp isOnSnowLeopardOrBetter]
+                ? [[NSPasteboard generalPasteboard] canReadObjectForClasses: [NSArray arrayWithObject: [NSString class]] options: nil]
+                : [[NSPasteboard generalPasteboard] availableTypeFromArray: [NSArray arrayWithObject: NSStringPboardType]] != nil);
+    
+    return YES;
+}
+
+- (void) paste: (id) sender
+{
+    NSMutableArray * tempTrackers = [NSMutableArray array];
+    
+    if ([NSApp isOnSnowLeopardOrBetter])
+    {
+        NSArray * items = [[NSPasteboard generalPasteboard] readObjectsForClasses:
+                            [NSArray arrayWithObject: [NSString class]] options: nil];
+        NSAssert(items != nil, @"no string items to paste; should not be able to call this method");
+        
+        for (NSString * pbItem in items)
+        {
+            for (NSString * tracker in [pbItem componentsSeparatedByString: @"\n"])
+                [tempTrackers addObject: tracker];
+        }
+    }
+    else
+    {
+        NSString * pbItem =[[NSPasteboard generalPasteboard] stringForType: NSStringPboardType];
+        NSAssert(pbItem != nil, @"no string items to paste; should not be able to call this method");
+        
+        for (NSString * tracker in [pbItem componentsSeparatedByString: @"\n"])
+            [tempTrackers addObject: tracker];
+    }
+    
+    BOOL added = NO;
+    
+    for (NSString * tracker in tempTrackers)
+    {
+        tracker = [tracker stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+        
+        if ([tracker rangeOfString: @"://"].location == NSNotFound)
+            tracker = [@"http://" stringByAppendingString: tracker];
+        
+        if (tr_urlIsValidTracker([tracker UTF8String]))
+        {
+            [fTrackers addObject: tracker];
+            added = YES;
+        }
+    }
+    
+    if (added)
+    {
+        [fTrackerTable deselectAll: self];
+        [fTrackerTable reloadData];
+    }
+    else
+        NSBeep();
+}
+
 @end
 
 @implementation CreatorWindowController (Private)
