@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: FileNameCell.m 10206 2010-02-15 14:56:14Z livings124 $
+ * $Id: FileNameCell.m 11028 2010-07-21 03:59:08Z livings124 $
  * 
  * Copyright (c) 2007-2010 Transmission authors and contributors
  *
@@ -29,6 +29,9 @@
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 
+#import "transmission.h" // required by utils.h
+#import "utils.h"
+
 #define PADDING_HORIZONAL 2.0
 #define IMAGE_FOLDER_SIZE 16.0
 #define IMAGE_ICON_SIZE 32.0
@@ -54,17 +57,21 @@
     if ((self = [super init]))
     {
         NSMutableParagraphStyle * paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
-        [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
+        [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingMiddle];
         
         fTitleAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
                             [NSFont messageFontOfSize: 12.0], NSFontAttributeName,
                             paragraphStyle, NSParagraphStyleAttributeName, nil];
         
+        NSMutableParagraphStyle * statusParagraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
+        [statusParagraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
+        
         fStatusAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
                                 [NSFont messageFontOfSize: 9.0], NSFontAttributeName,
-                                paragraphStyle, NSParagraphStyleAttributeName, nil];
+                                statusParagraphStyle, NSParagraphStyleAttributeName,  nil];
         
         [paragraphStyle release];
+        [statusParagraphStyle release];
     }
     return self;
 }
@@ -157,14 +164,15 @@
     {
         result.origin.x = NSMinX(bounds) + PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
         result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE_FILE;
+        result.size.width = NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONAL;
     }
     else
     {
         result.origin.x = NSMinX(bounds) + PADDING_HORIZONAL + IMAGE_FOLDER_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
         result.origin.y = NSMidY(bounds) - titleSize.height * 0.5;
+        result.size.width = MIN(titleSize.width, NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONAL);
     }
     result.size.height = titleSize.height;
-    result.size.width = MIN(titleSize.width, NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONAL);
     
     return result;
 }
@@ -176,17 +184,17 @@
     NSRect result;
     if (![(FileListNode *)[self objectValue] isFolder])
     {
-        result.origin.x = NSMinX(bounds) + PADDING_HORIZONAL + IMAGE_ICON_SIZE + PADDING_BETWEEN_IMAGE_AND_TITLE;
+        result.origin.x = NSMinX(titleRect);
         result.origin.y = NSMaxY(bounds) - PADDING_BELOW_STATUS_FILE - statusSize.height;
+        result.size.width = NSWidth(titleRect);
     }
     else
     {
         result.origin.x = NSMaxX(titleRect) + PADDING_BETWEEN_NAME_AND_FOLDER_STATUS;
         result.origin.y = NSMaxY(titleRect) - statusSize.height - 1.0;
+        result.size.width = NSMaxX(bounds) - NSMaxX(titleRect) - PADDING_HORIZONAL;
     }
-        
     result.size.height = statusSize.height;
-    result.size.width = NSMaxX(bounds) - NSMaxX(result) - PADDING_HORIZONAL;
     
     return result;
 }
@@ -203,7 +211,8 @@
     FileListNode * node = (FileListNode *)[self objectValue];
     
     const CGFloat progress = [torrent fileProgress: node];
-    NSString * percentString = progress == 1.0 ? @"100%" : [NSString localizedStringWithFormat: @"%.2f%%", progress * 100.0];
+    NSString * percentString = progress == 1.0 ? @"100%"
+                                : [NSString localizedStringWithFormat: @"%.2f%%", tr_truncd(progress * 100.0, 2)];
     
     NSString * status = [NSString stringWithFormat: NSLocalizedString(@"%@ of %@",
                             "Inspector -> Files tab -> file status string"), percentString, [NSString stringForFileSize: [node size]]];
