@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: GroupsController.m 9844 2010-01-01 21:12:04Z livings124 $
+ * $Id: GroupsController.m 10448 2010-04-06 01:41:16Z livings124 $
  *
  * Copyright (c) 2007-2010 Transmission authors and contributors
  *
@@ -251,23 +251,14 @@ GroupsController * fGroupsInstance = nil;
 - (void) addNewGroup
 {
     //find the lowest index
-    NSInteger index;
-    for (index = 0; index < [fGroups count]; index++)
-    {
-        BOOL found = NO;
-        for (NSDictionary * dict in fGroups)
-            if ([[dict objectForKey: @"Index"] integerValue] == index)
-            {
-                found = YES;
-                break;
-            }
-        
-        if (!found)
-            break;
-    }
+    NSMutableIndexSet * candidates = [NSMutableIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fGroups count]+1)];
+    for (NSDictionary * dict in fGroups)
+        [candidates removeIndex: [[dict objectForKey: @"Index"] integerValue]];
+    
+    const NSInteger index = [candidates firstIndex];
     
     [fGroups addObject: [NSMutableDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithInteger: index], @"Index",
-                            [NSColor cyanColor], @"Color", @"", @"Name", nil]];
+                            [NSColor colorWithCalibratedRed: 0.0 green: 0.65 blue: 1.0 alpha: 1.0], @"Color", @"", @"Name", nil]];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGroups" object: self];
     [self saveGroups];
