@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: InfoTabButtonCell.m 9887 2010-01-05 04:34:31Z livings124 $
+ * $Id: InfoTabButtonCell.m 10376 2010-03-14 15:03:14Z livings124 $
  *
  * Copyright (c) 2007-2010 Transmission authors and contributors
  *
@@ -35,6 +35,10 @@
         name: NSControlTintDidChangeNotification object: NSApp];
     
     fSelected = NO;
+    
+    //expects the icon to currently be set as the image
+    fIcon = [[self image] retain];
+    [self setSelectedTab: fSelected];
 }
 
 - (void) dealloc
@@ -45,14 +49,6 @@
     [super dealloc];
 }
 
-- (void) setIcon: (NSImage *) image
-{
-    [fIcon release];
-    fIcon = [image retain];
-    
-    [self setSelectedTab: fSelected];
-}
-
 - (void) setSelectedTab: (BOOL) selected
 {
     fSelected = selected;
