@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: MessageWindowController.m 10236 2010-02-20 14:29:33Z livings124 $
+ * $Id: MessageWindowController.m 10322 2010-03-07 14:39:13Z livings124 $
  *
  * Copyright (c) 2006-2010 Transmission authors and contributors
  *
@@ -75,11 +75,6 @@
     
     [window setContentBorderThickness: NSMinY([[fMessageTable enclosingScrollView] frame]) forEdge: NSMinYEdge];
     
-    //initially sort peer table by date
-    if ([[fMessageTable sortDescriptors] count] == 0)
-        [fMessageTable setSortDescriptors: [NSArray arrayWithObject: [[fMessageTable tableColumnWithIdentifier: @"Date"]
-                                            sortDescriptorPrototype]]];
-    
     [[self window] setTitle: NSLocalizedString(@"Message Log", "Message window -> title")];
     
     //set images and text for popup button items
