@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PeerProgressIndicatorCell.m 6980 2008-10-29 02:33:20Z livings124 $
+ * $Id: PeerProgressIndicatorCell.m 10711 2010-05-31 14:06:54Z livings124 $
  * 
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,10 @@
  *****************************************************************************/
 
 #import "PeerProgressIndicatorCell.h"
+#import "NSApplicationAdditions.h"
+
+#import "transmission.h" // required by utils.h
+#import "utils.h"
 
 @implementation PeerProgressIndicatorCell
 
@@ -40,13 +44,27 @@
     [super dealloc];
 }
 
+- (void) setSeed: (BOOL) seed
+{
+    fSeed = seed;
+}
+
 - (void) drawWithFrame: (NSRect) cellFrame inView: (NSView *) controlView
 {
     if ([[NSUserDefaults standardUserDefaults] boolForKey: @"DisplayPeerProgressBarNumber"])
     {
         if (!fAttributes)
-            fAttributes = [[NSDictionary alloc] initWithObjectsAndKeys: [NSFont systemFontOfSize: 11.0f], NSFontAttributeName, nil];
-        [[NSString localizedStringWithFormat: @"%.1f%%", [self floatValue] * 100.0f] drawInRect: cellFrame withAttributes: fAttributes];
+        {
+            NSMutableParagraphStyle * paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
+            [paragraphStyle setAlignment: NSRightTextAlignment];
+            
+            fAttributes = [[NSDictionary alloc] initWithObjectsAndKeys: [NSFont systemFontOfSize: 11.0], NSFontAttributeName,
+                                                                            paragraphStyle, NSParagraphStyleAttributeName, nil];
+            [paragraphStyle release];
+        }
+        
+        [[NSString localizedStringWithFormat: @"%.1f%%", tr_truncd([self floatValue] * 100.0, 1)] drawInRect: cellFrame
+            withAttributes: fAttributes];
     }
     else
     {
@@ -58,15 +76,22 @@
         }
         
         [super drawWithFrame: cellFrame inView: controlView];
-        if ([self floatValue] >= 1.0f)
+        if (fSeed)
         {
             NSImage * checkImage = [NSImage imageNamed: @"CompleteCheck.png"];
-            [checkImage setFlipped: YES];
             
-            NSSize imageSize = [checkImage size];
-            NSRect rect = NSMakeRect(cellFrame.origin.x + (cellFrame.size.width - imageSize.width) * 0.5f,
-                        cellFrame.origin.y + (cellFrame.size.height - imageSize.height) * 0.5f, imageSize.width, imageSize.height);
-            [checkImage drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0f];
+            const NSSize imageSize = [checkImage size];
+            const NSRect rect = NSMakeRect(NSMidX(cellFrame) - imageSize.width * 0.5, NSMidY(cellFrame) - imageSize.height * 0.5,
+                                            imageSize.width, imageSize.height);
+            
+            if ([NSApp isOnSnowLeopardOrBetter])
+                [checkImage drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0 respectFlipped: YES
+                        hints: nil];
+            else
+            {
+                [checkImage setFlipped: YES];
+                [checkImage drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+            }
         }
     }
 }
