@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.m 10119 2010-02-06 18:44:13Z livings124 $
+ * $Id: PrefsController.m 10833 2010-06-24 01:19:29Z livings124 $
  *
  * Copyright (c) 2005-2010 Transmission authors and contributors
  *
@@ -30,6 +30,8 @@
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
+
+#import "transmission.h"
 #import "utils.h"
 
 #import <Sparkle/Sparkle.h>
@@ -479,6 +481,11 @@ tr_session * fHandle;
     tr_sessionSetDHTEnabled(fHandle, [fDefaults boolForKey: @"DHTGlobal"]);
 }
 
+- (void) setLPD: (id) sender
+{
+    tr_sessionSetLPDEnabled(fHandle, [fDefaults boolForKey: @"LocalPeerDiscoveryGlobal"]);
+}
+
 - (void) setEncryptionMode: (id) sender
 {
     const tr_encryption_mode mode = [fDefaults boolForKey: @"EncryptionPrefer"] ? 
@@ -556,6 +563,11 @@ tr_session * fHandle;
         NSLocalizedString(@"Last updated", "Prefs -> blocklist -> message"), updatedDateString]];
 }
 
+- (void) setAutoStartDownloads: (id) sender
+{
+    tr_sessionSetPaused(fHandle, ![fDefaults boolForKey: @"AutoStartDownload"]);
+}
+
 - (void) applySpeedSettings: (id) sender
 {
     tr_sessionLimitSpeed(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
@@ -1091,19 +1103,19 @@ tr_session * fHandle;
 
 - (void) helpForPeers: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"PeersPrefs"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"peers"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
 - (void) helpForNetwork: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"NetworkPrefs"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"network"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
 - (void) helpForRemote: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"RemotePrefs"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"remote"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
@@ -1142,6 +1154,14 @@ tr_session * fHandle;
     const BOOL dht = tr_sessionIsDHTEnabled(fHandle);
     [fDefaults setBool: dht forKey: @"DHTGlobal"];
     
+    //dht
+    const BOOL lpd = tr_sessionIsLPDEnabled(fHandle);
+    [fDefaults setBool: lpd forKey: @"LocalPeerDiscovery"];
+    
+    //auto start
+    const BOOL autoStart = !tr_sessionGetPaused(fHandle);
+    [fDefaults setBool: autoStart forKey: @"AutoStartDownload"];
+    
     //port
     const tr_port port = tr_sessionGetPeerPort(fHandle);
     [fDefaults setInteger: port forKey: @"BindPort"];
@@ -1220,6 +1240,8 @@ tr_session * fHandle;
         
         //dht handled by bindings
         
+        //lpd handled by bindings
+        
         [fPortField setIntValue: port];
         //port forwarding (nat) handled by bindings
         //random port handled by bindings
