@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentCell.h 8260 2009-04-20 02:45:27Z livings124 $
+ * $Id: TorrentCell.h 10405 2010-03-20 18:09:45Z livings124 $
  *
- * Copyright (c) 2006-2009 Transmission authors and contributors
+ * Copyright (c) 2006-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,31 +25,20 @@
 @interface TorrentCell : NSActionCell
 {
     NSUserDefaults * fDefaults;
-    NSImage * fErrorImage;
     
     NSMutableDictionary * fTitleAttributes, * fStatusAttributes;
     
     BOOL fTracking, fMouseDownControlButton, fMouseDownRevealButton, fMouseDownActionButton,
-            fHoverControl, fHoverReveal, fHoverAction;
+            fHover, fHoverControl, fHoverReveal, fHoverAction;
     
-    NSImage * fHighPriorityImage, * fLowPriorityImage;
-    
-    NSColor * fBarBorderColor, * fBluePieceColor;
+    NSColor * fBarBorderColor, * fBluePieceColor, * fBarMinimalBorderColor;
 }
 
 - (NSRect) iconRectForBounds: (NSRect) bounds;
-- (NSRect) titleRectForBounds: (NSRect) bounds;
-- (NSRect) progressRectForBounds: (NSRect) bounds;
-- (NSRect) barRectForBounds: (NSRect) bounds;
-- (NSRect) statusRectForBounds: (NSRect) bounds;
-- (NSRect) minimalStatusRectForBounds: (NSRect) bounds;
-
-- (NSRect) controlButtonRectForBounds: (NSRect) bounds;
-- (NSRect) revealButtonRectForBounds: (NSRect) bounds;
-- (NSRect) actionButtonRectForBounds: (NSRect) bounds;
 
 - (void) addTrackingAreasForView: (NSView *) controlView inRect: (NSRect) cellFrame withUserInfo: (NSDictionary *) userInfo
             mouseLocation: (NSPoint) mouseLocation;
+- (void) setHover: (BOOL) hover;
 - (void) setControlHover: (BOOL) hover;
 - (void) setRevealHover: (BOOL) hover;
 - (void) setActionHover: (BOOL) hover;
