@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: app.h 9868 2010-01-04 21:00:47Z charles $
+ * $Id: app.h 10775 2010-06-16 03:11:10Z Longinus00 $
  */
 
 #ifndef QTR_APP_H
@@ -30,6 +30,9 @@ class MyApp: public QApplication
         MyApp( int& argc, char ** argv );
         virtual ~MyApp( );
 
+    public:
+        void raise( );
+
     private:
         Prefs * myPrefs;
         Session * mySession;
@@ -43,8 +46,11 @@ class MyApp: public QApplication
 
     private slots:
         void consentGiven( );
+        void onSessionSourceChanged( );
         void refreshPref( int key );
         void refreshTorrents( );
+
+    public slots:
         void addTorrent( const QString& );
 
     private:
