@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: mainwin.cc 9890 2010-01-05 23:47:50Z charles $
+ * $Id: mainwin.cc 10871 2010-06-26 17:20:22Z Longinus00 $
  */
 
 #include <cassert>
@@ -20,6 +20,7 @@
 #include <QHBoxLayout>
 #include <QInputDialog>
 #include <QLabel>
+#include <QMessageBox>
 #include <QSignalMapper>
 #include <QSize>
 #include <QStyle>
@@ -85,11 +86,10 @@ namespace
 TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& model, bool minimized ):
     myLastFullUpdateTime( 0 ),
     mySessionDialog( new SessionDialog( session, prefs, this ) ),
-    myPrefsDialog( new PrefsDialog( session, prefs, this ) ),
+    myPrefsDialog( 0 ),
     myAboutDialog( new AboutDialog( this ) ),
     myStatsDialog( new StatsDialog( session, this ) ),
     myDetailsDialog( 0 ),
-    myFileDialog( 0 ),
     myFilterModel( prefs ),
     myTorrentDelegate( new TorrentDelegate( this ) ),
     myTorrentDelegateMin( new TorrentDelegateMin( this ) ),
@@ -136,7 +136,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_TrayIcon, SIGNAL(toggled(bool)), this, SLOT(setTrayIconVisible(bool)));
     connect( ui.action_Filterbar, SIGNAL(toggled(bool)), this, SLOT(setFilterbarVisible(bool)));
     connect( ui.action_Statusbar, SIGNAL(toggled(bool)), this, SLOT(setStatusbarVisible(bool)));
-    connect( ui.action_MinimalView, SIGNAL(toggled(bool)), this, SLOT(setMinimalView(bool)));
+    connect( ui.action_CompactView, SIGNAL(toggled(bool)), this, SLOT(setCompactView(bool)));
     connect( ui.action_SortByActivity, SIGNAL(toggled(bool)), this, SLOT(onSortByActivityToggled(bool)));
     connect( ui.action_SortByAge,      SIGNAL(toggled(bool)), this, SLOT(onSortByAgeToggled(bool)));
     connect( ui.action_SortByETA,      SIGNAL(toggled(bool)), this, SLOT(onSortByETAToggled(bool)));
@@ -158,7 +158,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_AddFile, SIGNAL(triggered()), this, SLOT(openTorrent()));
     connect( ui.action_AddURL, SIGNAL(triggered()), this, SLOT(openURL()));
     connect( ui.action_New, SIGNAL(triggered()), this, SLOT(newTorrent()));
-    connect( ui.action_Preferences, SIGNAL(triggered()), myPrefsDialog, SLOT(show()));
+    connect( ui.action_Preferences, SIGNAL(triggered()), this, SLOT(openPreferences()));
     connect( ui.action_Statistics, SIGNAL(triggered()), myStatsDialog, SLOT(show()));
     connect( ui.action_About, SIGNAL(triggered()), myAboutDialog, SLOT(show()));
     connect( ui.action_Contents, SIGNAL(triggered()), this, SLOT(openHelp()));
@@ -259,7 +259,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
              << Prefs :: STATUSBAR_STATS
              << Prefs :: TOOLBAR
              << Prefs :: ALT_SPEED_LIMIT_ENABLED
-             << Prefs :: MINIMAL_VIEW
+             << Prefs :: COMPACT_VIEW
              << Prefs :: DSPEED
              << Prefs :: DSPEED_ENABLED
              << Prefs :: USPEED
@@ -438,8 +438,8 @@ QWidget *
 TrMainWindow :: createStatusBar( )
 {
     QMenu * m;
-    QLabel *l, *l2;
-    QWidget *w;
+    QLabel * l;
+    QWidget * w;
     QHBoxLayout * h;
     QPushButton * p;
     QActionGroup * a;
@@ -453,12 +453,16 @@ TrMainWindow :: createStatusBar( )
 
         p = myOptionsButton = new TrIconPushButton( this );
         p->setIcon( QIcon( ":/icons/options.png" ) );
+        p->setIconSize( QPixmap( ":/icons/options.png" ).size() );
         p->setFlat( true );
         p->setMenu( createOptionsMenu( ) );
         h->addWidget( p );
 
-        p = myAltSpeedButton = new TrIconPushButton( this );
+        p = myAltSpeedButton = new QPushButton( this );
         p->setIcon( myPrefs.get<bool>(Prefs::ALT_SPEED_LIMIT_ENABLED) ? mySpeedModeOnIcon : mySpeedModeOffIcon );
+        p->setIconSize( QPixmap( ":/icons/alt-limit-on.png" ).size() );
+        p->setCheckable( true );
+        p->setFixedWidth( p->height() );
         p->setFlat( true );
         h->addWidget( p );
         connect( p, SIGNAL(clicked()), this, SLOT(toggleSpeedMode()));
@@ -489,33 +493,36 @@ TrMainWindow :: createStatusBar( )
         connect( ui.action_SessionTransfer, SIGNAL(triggered()), this, SLOT(showSessionTransfer()));
         p = myStatsModeButton = new TrIconPushButton( this );
         p->setIcon( QIcon( ":/icons/ratio.png" ) );
+        p->setIconSize( QPixmap( ":/icons/ratio.png" ).size() );
         p->setFlat( true );
         p->setMenu( m );
         h->addWidget( p );
         l = myStatsLabel = new QLabel( this );
         h->addWidget( l );
 
-        w = new QWidget( this );
-        w->setMinimumSize( HIG::PAD_BIG, 1 );
-        w->setMaximumSize( HIG::PAD_BIG, 1 );
-        h->addWidget( w );
-        l = new QLabel( this );
-        l->setPixmap( getStockIcon( "go-down", QStyle::SP_ArrowDown ).pixmap( smallIconSize ) );
-        h->addWidget( l );
-        l2 = myDownloadSpeedLabel = new QLabel( this );
-        h->addWidget( l2 );
-        myDownStatusWidgets << w << l << l2;
+    h->addSpacing( HIG::PAD_BIG );
 
         w = new QWidget( this );
         w->setMinimumSize( HIG::PAD_BIG, 1 );
         w->setMaximumSize( HIG::PAD_BIG, 1 );
         h->addWidget( w );
+        l = myDownloadSpeedLabel = new QLabel( this );
+        h->addWidget( l );
+        l = new QLabel( this );
+        l->setPixmap( getStockIcon( "go-down", QStyle::SP_ArrowDown ).pixmap( smallIconSize ) );
+        h->addWidget( l );
+
+    h->addSpacing( HIG::PAD_BIG );
+
+        w = new QWidget( this );
+        w->setMinimumSize( HIG::PAD_BIG, 1 );
+        w->setMaximumSize( HIG::PAD_BIG, 1 );
+        h->addWidget( w );
+        l = myUploadSpeedLabel = new QLabel;
+        h->addWidget( l );
         l = new QLabel;
         l->setPixmap( getStockIcon( "go-up", QStyle::SP_ArrowUp ).pixmap( smallIconSize ) );
         h->addWidget( l );
-        l2 = myUploadSpeedLabel = new QLabel;
-        h->addWidget( l2 );
-        myUpStatusWidgets << w << l << l2;
 
     return top;
 }
@@ -628,6 +635,23 @@ TrMainWindow :: setSortAscendingPref( bool b )
 *****
 ****/
 
+void
+TrMainWindow :: onPrefsDestroyed( )
+{
+    myPrefsDialog = 0;
+}
+
+void
+TrMainWindow :: openPreferences( )
+{
+    if( myPrefsDialog == 0 ) {
+        myPrefsDialog = new PrefsDialog( mySession, myPrefs, this );
+        connect( myPrefsDialog, SIGNAL(destroyed(QObject*)), this, SLOT(onPrefsDestroyed()));
+    }
+
+    myPrefsDialog->show( );
+}
+
 void
 TrMainWindow :: onDetailsDestroyed( )
 {
@@ -638,7 +662,7 @@ void
 TrMainWindow :: openProperties( )
 {
     if( myDetailsDialog == 0 ) {
-        myDetailsDialog = new Details( mySession, myModel, this );
+        myDetailsDialog = new Details( mySession, myPrefs, myModel, this );
         connect( myDetailsDialog, SIGNAL(destroyed(QObject*)), this, SLOT(onDetailsDestroyed()));
     }
 
@@ -649,7 +673,7 @@ TrMainWindow :: openProperties( )
 void
 TrMainWindow :: setLocation( )
 {
-    QDialog * d = new RelocateDialog( mySession, getSelectedTorrents(), this );
+    QDialog * d = new RelocateDialog( mySession, myModel, getSelectedTorrents(), this );
     d->show( );
 }
 
@@ -686,7 +710,7 @@ TrMainWindow :: refreshTitle( )
     QString title( "Transmission" );
     const QUrl url( mySession.getRemoteUrl( ) );
     if( !url.isEmpty() )
-        title += tr( " - %1" ).arg( url.toString(QUrl::RemoveUserInfo) );
+        title += tr( " - %1:%2" ).arg( url.host() ).arg( url.port() );
     setWindowTitle( title );
 }
 
@@ -711,8 +735,6 @@ TrMainWindow :: refreshStatusBar( )
     const Speed down( myModel.getDownloadSpeed( ) );
     myUploadSpeedLabel->setText( Utils :: speedToString( up ) );
     myDownloadSpeedLabel->setText( Utils :: speedToString( down ) );
-    foreach( QWidget * w, myUpStatusWidgets ) w->setVisible( !up.isZero( ) );
-    foreach( QWidget * w, myDownStatusWidgets ) w->setVisible( !down.isZero( ) );
 
     myNetworkLabel->setVisible( !mySession.isServer( ) );
 
@@ -843,12 +865,12 @@ TrMainWindow :: pauseAll( )
 void
 TrMainWindow :: removeSelected( )
 {
-    mySession.removeTorrents( getSelectedTorrents( ), false );
+    removeTorrents( false );
 }
 void
 TrMainWindow :: deleteSelected( )
 {
-    mySession.removeTorrents( getSelectedTorrents( ), true );
+    removeTorrents( true );
 }
 void
 TrMainWindow :: verifySelected( )
@@ -886,9 +908,9 @@ void TrMainWindow :: showSessionTransfer ( ) { myPrefs.set( Prefs::STATUSBAR_STA
 **/
 
 void
-TrMainWindow :: setMinimalView( bool visible )
+TrMainWindow :: setCompactView( bool visible )
 {
-    myPrefs.set( Prefs :: MINIMAL_VIEW, visible );
+    myPrefs.set( Prefs :: COMPACT_VIEW, visible );
 }
 void
 TrMainWindow :: setTrayIconVisible( bool visible )
@@ -1038,9 +1060,9 @@ TrMainWindow :: refreshPref( int key )
             myTrayIcon.setVisible( b );
             break;
 
-        case Prefs::MINIMAL_VIEW:
+        case Prefs::COMPACT_VIEW:
             b = myPrefs.getBool( key );
-            ui.action_MinimalView->setChecked( b );
+            ui.action_CompactView->setChecked( b );
             ui.listView->setItemDelegate( b ? myTorrentDelegateMin : myTorrentDelegate );
             ui.listView->reset( ); // force the rows to resize
             break;
@@ -1089,37 +1111,41 @@ TrMainWindow :: newTorrent( )
 void
 TrMainWindow :: openTorrent( )
 {
-    if( myFileDialog == 0 )
-    {
-        myFileDialog = new QFileDialog( this,
-                                        tr( "Add Torrent" ),
-                                        myPrefs.getString( Prefs::OPEN_DIALOG_FOLDER ),
-                                        tr( "Torrent Files (*.torrent);;All Files (*.*)" ) );
-        myFileDialog->setFileMode( QFileDialog::ExistingFiles );
+    QFileDialog * myFileDialog;
+    myFileDialog = new QFileDialog( this,
+                                    tr( "Add Torrent" ),
+                                    myPrefs.getString( Prefs::OPEN_DIALOG_FOLDER ),
+                                    tr( "Torrent Files (*.torrent);;All Files (*.*)" ) );
+    myFileDialog->setFileMode( QFileDialog::ExistingFiles );
 
+    QCheckBox * button = new QCheckBox( tr( "Show &options dialog" ) );
+    button->setChecked( myPrefs.getBool( Prefs::OPTIONS_PROMPT ) );
+    QGridLayout * layout = dynamic_cast<QGridLayout*>(myFileDialog->layout());
+    layout->addWidget( button, layout->rowCount( ), 0, 1, -1, Qt::AlignLeft );
+    myFileDialogOptionsCheck = button;
 
-        QCheckBox * button = new QCheckBox( tr( "Show &options dialog" ) );
-        button->setChecked( myPrefs.getBool( Prefs::OPTIONS_PROMPT ) );
-        QGridLayout * layout = dynamic_cast<QGridLayout*>(myFileDialog->layout());
-        layout->addWidget( button, layout->rowCount( ), 0, 1, -1, Qt::AlignLeft );
-        myFileDialogOptionsCheck = button;
-
-        connect( myFileDialog, SIGNAL(filesSelected(const QStringList&)),
-                 this, SLOT(addTorrents(const QStringList&)));
-    }
+    connect( myFileDialog, SIGNAL(filesSelected(const QStringList&)),
+             this, SLOT(addTorrents(const QStringList&)));
 
     myFileDialog->show( );
 }
 
 void
 TrMainWindow :: openURL( )
+{
+    QString tmp;
+    openURL( tmp );
+}
+
+void
+TrMainWindow :: openURL( QString url )
 {
     bool ok;
     const QString key = QInputDialog::getText( this,
                                                tr( "Add URL or Magnet Link" ),
                                                tr( "Add URL or Magnet Link" ),
                                                QLineEdit::Normal,
-                                               QString( ),
+                                               url,
                                                &ok );
     if( ok && !key.isEmpty( ) )
         mySession.addTorrent( key );
@@ -1145,6 +1171,101 @@ TrMainWindow :: addTorrent( const QString& filename )
     }
 }
 
+void
+TrMainWindow :: removeTorrents( const bool deleteFiles )
+{
+    QSet<int> ids;
+    QMessageBox msgBox( this );
+    QString primary_text, secondary_text;
+    int incomplete = 0;
+    int connected  = 0;
+    int count;
+
+    foreach( QModelIndex index, ui.listView->selectionModel( )->selectedRows( ) )
+    {
+        const Torrent * tor( index.model()->data( index, TorrentModel::TorrentRole ).value<const Torrent*>( ) );
+        ids.insert( tor->id( ) );
+        if( tor->connectedPeers( ) )
+            ++connected;
+        if( !tor->isDone( ) )
+            ++incomplete;
+    }
+
+    if( ids.isEmpty() )
+        return;
+    count = ids.size();
+
+    if( !deleteFiles )
+    {
+        primary_text = ( count == 1 )
+            ? tr( "Remove torrent?" )
+            : tr( "Remove %1 torrents?" ).arg( count );
+    }
+    else
+    {
+        primary_text = ( count == 1 )
+            ? tr( "Delete this torrent's downloaded files?" )
+            : tr( "Delete these %1 torrents' downloaded files?" ).arg( count );
+    }
+
+    if( !incomplete && !connected )
+    {
+        secondary_text = ( count == 1 )
+            ? tr( "Once removed, continuing the transfer will require the torrent file or magnet link." )
+            : tr( "Once removed, continuing the transfers will require the torrent files or magnet links." );
+    }
+    else if( count == incomplete )
+    {
+        secondary_text = ( count == 1 )
+            ? tr( "This torrent has not finished downloading." )
+            : tr( "These torrents have not finished downloading." );
+    }
+    else if( count == connected )
+    {
+        secondary_text = ( count == 1 )
+            ? tr( "This torrent is connected to peers." )
+            : tr( "These torrents are connected to peers." );
+    }
+    else
+    {
+        if( connected )
+        {
+            secondary_text = ( connected == 1 )
+                ? tr( "One of these torrents is connected to peers." )
+                : tr( "Some of these torrents are connected to peers." );
+        }
+
+        if( connected && incomplete )
+        {
+            secondary_text += "\n";
+        }
+
+        if( incomplete )
+        {
+            secondary_text += ( incomplete == 1 )
+                ? tr( "One of these torrents has not finished downloading." )
+                : tr( "Some of these torrents have not finished downloading." );
+        }
+    }
+
+    msgBox.setWindowTitle( QString(" ") );
+    msgBox.setText( QString( "<big><b>%1</big></b>" ).arg( primary_text ) );
+    msgBox.setInformativeText( secondary_text );
+    msgBox.setStandardButtons( QMessageBox::Ok | QMessageBox::Cancel );
+    msgBox.setDefaultButton( QMessageBox::Cancel );
+    msgBox.setIcon( QMessageBox::Question );
+    /* hack needed to keep the dialog from being too narrow */
+    QGridLayout* layout = (QGridLayout*)msgBox.layout();
+    QSpacerItem* spacer = new QSpacerItem( 450, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
+    layout->addItem( spacer, layout->rowCount(), 0, 1, layout->columnCount() );
+
+    if( msgBox.exec() == QMessageBox::Ok )
+    {
+        ui.listView->selectionModel()->clear();
+        mySession.removeTorrents( ids, deleteFiles );
+    }
+}
+
 /***
 ****
 ***/
