@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: mainwin.h 9868 2010-01-04 21:00:47Z charles $
+ * $Id: mainwin.h 10871 2010-06-26 17:20:22Z Longinus00 $
  */
 
 #ifndef MAIN_WINDOW_H
@@ -15,11 +15,11 @@
 
 #include <ctime>
 #include <QCheckBox>
-#include <QFileDialog>
 #include <QLineEdit>
 #include <QIcon>
 #include <QMainWindow>
 #include <QMap>
+#include <QPushButton>
 #include <QSet>
 #include <QSystemTrayIcon>
 #include <QTimer>
@@ -60,7 +60,6 @@ class TrMainWindow: public QMainWindow
         QDialog * myAboutDialog;
         QDialog * myStatsDialog;
         Details * myDetailsDialog;
-        QFileDialog * myFileDialog;
         QCheckBox * myFileDialogOptionsCheck;
         QSystemTrayIcon myTrayIcon;
         TorrentFilter myFilterModel;
@@ -91,7 +90,12 @@ class TrMainWindow: public QMainWindow
         void updateNetworkIcon( );
         QWidgetList myHidden;
 
+    public slots:
+        void openURL( QString );
+
     private slots:
+        void onPrefsDestroyed( );
+        void openPreferences( );
         void onDetailsDestroyed( );
         void onShowModeClicked( );
         void showAll( );
@@ -115,6 +119,7 @@ class TrMainWindow: public QMainWindow
         void trayActivated( QSystemTrayIcon::ActivationReason );
         void refreshPref( int key );
         void addTorrents( const QStringList& filenames );
+        void removeTorrents( const bool deleteFiles );
         void openHelp( );
         void openFolder( );
         void copyMagnetLinkToClipboard( );
@@ -153,8 +158,6 @@ class TrMainWindow: public QMainWindow
         QMenu * createOptionsMenu( void );
         QWidget * createStatusBar( void );
         QWidget * myStatusBar;
-        QWidgetList myUpStatusWidgets;
-        QWidgetList myDownStatusWidgets;
         QPushButton * myAltSpeedButton;
         QPushButton * myOptionsButton;
         QLabel * myVisibleCountLabel;
@@ -184,7 +187,7 @@ class TrMainWindow: public QMainWindow
         void setFilterbarVisible( bool );
         void setStatusbarVisible( bool );
         void setTrayIconVisible( bool );
-        void setMinimalView( bool );
+        void setCompactView( bool );
         void refreshActionSensitivity( );
         void wrongAuthentication( );
 
