@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: relocate.h 9868 2010-01-04 21:00:47Z charles $
+ * $Id: relocate.h 10722 2010-06-04 01:00:27Z Longinus00 $
  */
 
 #ifndef RELOCATE_DIALOG_H
@@ -20,17 +20,20 @@
 class QPushButton;
 class QRadioButton;
 class Session;
+class Torrent;
+class TorrentModel;
 
 class RelocateDialog: public QDialog
 {
         Q_OBJECT
 
     private:
-        static QString myPath;
+        QString myPath;
         static bool myMoveFlag;
 
     private:
         Session & mySession;
+        TorrentModel& myModel;
         QSet<int> myIds;
         QPushButton * myDirButton;
         QRadioButton * myMoveRadio;
@@ -42,7 +45,7 @@ class RelocateDialog: public QDialog
         void onMoveToggled( bool );
 
     public:
-        RelocateDialog( Session&, const QSet<int>& ids, QWidget * parent = 0 );
+        RelocateDialog( Session&, TorrentModel&, const QSet<int>& ids, QWidget * parent = 0 );
         ~RelocateDialog( ) { }
 };
 
