@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.h 9868 2010-01-04 21:00:47Z charles $
+ * $Id: utils.h 10722 2010-06-04 01:00:27Z Longinus00 $
  */
 
 #ifndef QTR_UTILS
@@ -27,7 +27,7 @@ class Utils: public QObject
         Utils( ) { }
         virtual ~Utils( ) { }
     public:
-
+        static QString remoteFileChooser( QWidget * parent, const QString& title, const QString& myPath, bool dir, bool local );
         static QString sizeToString( double size );
         static QString speedToString( const Speed& speed );
         static QString ratioToString( double ratio );
@@ -37,6 +37,16 @@ class Utils: public QObject
         // meh
         static void toStderr( const QString& qstr );
 
+        /// 
+        /// URLs
+        ///
+
+        static bool isMagnetLink( const QString& s ) { return s.startsWith( "magnet:?" ); }
+
+        static bool isURL( const QString& s ) { return s.startsWith( "http://" )
+                                                    || s.startsWith( "https://" )
+                                                    || s.startsWith( "ftp://" ); }
+
 };
 
 #endif
